# -*- coding: utf-8 -*-
"""The SHA-1 Hasher implementation"""

import hashlib

from plaso.analyzers.hashers import interface
from plaso.analyzers.hashers import manager


class SHA1Hasher(interface.BaseHasher):
  """This class provides SHA-1 hashing functionality."""

  NAME = u'sha1'
  DESCRIPTION = u'Calculates a SHA-1 digest hash over input data.'

  def __init__(self):
    """Initializes the SHA-1 hasher."""
    super(SHA1Hasher, self).__init__()
    self._sha1_context = hashlib.sha1()

  def GetBinaryDigest(self):
    """Returns the digest of the hash function as a binary string.

    Returns:
      bytes: binary string hash digest calculated over the data blocks passed to
          Update().
    """
    return self._sha1_context.digest()

  def GetStringDigest(self):
    """Returns the digest of the hash function expressed as a Unicode string.

    Returns:
      str: string hash digest calculated over the data blocks passed to
          Update(). The string consists of printable Unicode characters.
    """
    return u'{0:s}'.format(self._sha1_context.hexdigest())

  def Update(self, data):
    """Updates the current state of the hasher with a new block of data.

    Repeated calls to update are equivalent to one single call with the
    concatenation of the arguments.

    Args:
      data(bytes): block of data with which to update the context of the hasher.
    """
    self._sha1_context.update(data)


manager.HashersManager.RegisterHasher(SHA1Hasher)



# -*- coding: utf-8 -*-
"""The psort multi-processing engine."""

import collections
import heapq
import logging
import os
import time

from plaso.engine import plaso_queue
from plaso.engine import zeromq_queue
from plaso.containers import tasks
from plaso.lib import bufferlib
from plaso.lib import definitions
from plaso.lib import py2to3
from plaso.multi_processing import analysis_process
from plaso.multi_processing import engine as multi_process_engine
from plaso.multi_processing import multi_process_queue
from plaso.storage import time_range as storage_time_range


class PsortEventHeap(object):
  """Psort event heap."""

  _IDENTIFIER_EXCLUDED_ATTRIBUTES = frozenset([
      u'data_type',
      u'display_name',
      u'filename',
      u'inode',
      u'parser',
      u'tag',
      u'timestamp',
      u'timestamp_desc'])

  def __init__(self):
    """Initializes a psort events heap."""
    super(PsortEventHeap, self).__init__()
    self._heap = []

  @property
  def number_of_events(self):
    """int: number of events on the heap."""
    return len(self._heap)

  def _GetEventIdentifiers(self, event):
    """Retrieves different identifiers of the event.

    Every event contains event data, which consists of attributes and values.
    These attributes and values can be represented as a string and used for
    sorting and uniquely identifying events. This function determines multiple
    identifiers:
    * an identifier of the attributes and values without the timestamp
      description (or usage). This is referred to as the MACB group
      identifier.
    * an identifier of the attributes and values including the timestamp
      description (or usage). This is referred to as the event content
      identifier.

    The identifier without the timestamp description can be used to group
    events that have the same MACB (modification, access, change, birth)
    timestamps. The PsortEventHeap will store these events individually and
    relies on PsortMultiProcessEngine to do the actual grouping of events.

    Args:
      event (EventObject): event.

    Returns:
      tuple: contains:

        str: identifier of the event MACB group or None if the event cannot
            be grouped.
        str: identifier of the event content.
    """
    attributes = []

    attribute_string = u'data_type: {0:s}'.format(event.data_type)
    attributes.append(attribute_string)

    for attribute_name, attribute_value in sorted(event.GetAttributes()):
      if attribute_name in self._IDENTIFIER_EXCLUDED_ATTRIBUTES:
        continue

      if not attribute_value:
        continue

      if attribute_name == u'pathspec':
        attribute_value = attribute_value.comparable

      elif isinstance(attribute_value, dict):
        attribute_value = sorted(attribute_value.items())

      elif isinstance(attribute_value, set):
        attribute_value = sorted(list(attribute_value))

      elif isinstance(attribute_value, py2to3.BYTES_TYPE):
        attribute_value = repr(attribute_value)

      try:
        attribute_string = u'{0:s}: {1!s}'.format(
            attribute_name, attribute_value)
      except UnicodeDecodeError:
        logging.error(u'Failed to decode attribute {0:s}'.format(
            attribute_name))
      attributes.append(attribute_string)

    # The u'atime', u'ctime', u'crtime', u'mtime' are included for backwards
    # compatibility with the filestat parser.
    if event.timestamp_desc in (
        u'atime', u'ctime', u'crtime', u'mtime',
        definitions.TIME_DESCRIPTION_LAST_ACCESS,
        definitions.TIME_DESCRIPTION_CHANGE,
        definitions.TIME_DESCRIPTION_CREATION,
        definitions.TIME_DESCRIPTION_MODIFICATION):
      macb_group_identifier = u', '.join(attributes)
    else:
      macb_group_identifier = None

    attributes.insert(0, event.timestamp_desc)
    content_identifier = u', '.join(attributes)

    return macb_group_identifier, content_identifier

  def PopEvent(self):
    """Pops an event from the heap.

    Returns:
      tuple: contains:

        str: identifier of the event MACB group or None if the event cannot
            be grouped.
        str: identifier of the event content.
        EventObject: event.
    """
    try:
      return heapq.heappop(self._heap)

    except IndexError:
      return None

  def PopEvents(self):
    """Pops events from the heap.

    Yields:
      EventObject: event.
    """
    event = self.PopEvent()
    while event:
      yield event
      event = self.PopEvent()

  def PushEvent(self, event):
    """Pushes an event onto the heap.

    Args:
      event (EventObject): event.
    """
    macb_group_identifier, content_identifier = self._GetEventIdentifiers(event)

    # We can ignore the timestamp here because the psort engine only stores
    # events with the same timestamp in the event heap.
    heap_values = (macb_group_identifier, content_identifier, event)
    heapq.heappush(self._heap, heap_values)


class PsortMultiProcessEngine(multi_process_engine.MultiProcessEngine):
  """Psort multi-processing engine."""

  _PROCESS_JOIN_TIMEOUT = 5.0
  _PROCESS_WORKER_TIMEOUT = 15.0 * 60.0

  _QUEUE_TIMEOUT = 10 * 60

  def __init__(self, use_zeromq=True):
    """Initializes an engine object.

    Args:
      use_zeromq (Optional[bool]): True if ZeroMQ should be used for queuing
          instead of Python's multiprocessing queue.
    """
    super(PsortMultiProcessEngine, self).__init__()
    self._analysis_plugins = {}
    self._completed_analysis_processes = set()
    self._data_location = None
    self._event_filter_expression = None
    self._event_queues = {}
    # The event heap is used to make sure the events are sorted in
    # a deterministic way.
    self._export_event_heap = PsortEventHeap()
    self._export_event_timestamp = 0
    self._knowledge_base = None
    self._merge_task = None
    self._number_of_consumed_errors = 0
    self._number_of_consumed_event_tags = 0
    self._number_of_consumed_events = 0
    self._number_of_consumed_reports = 0
    self._number_of_consumed_sources = 0
    self._number_of_duplicate_events = 0
    self._number_of_macb_grouped_events = 0
    self._number_of_produced_errors = 0
    self._number_of_produced_event_tags = 0
    self._number_of_produced_events = 0
    self._number_of_produced_reports = 0
    self._number_of_produced_sources = 0
    self._status = definitions.PROCESSING_STATUS_IDLE
    self._status_update_callback = None
    self._use_zeromq = use_zeromq
    self._worker_memory_limit = definitions.DEFAULT_WORKER_MEMORY_LIMIT

  def _AnalyzeEvents(self, storage_writer, analysis_plugins, event_filter=None):
    """Analyzes events in a plaso storage.

    Args:
      storage_writer (StorageWriter): storage writer.
      analysis_plugins (dict[str, AnalysisPlugin]): analysis plugins that
          should be run and their names.
      event_filter (Optional[FilterObject]): event filter.

    Raises:
      RuntimeError: if a non-recoverable situation is encountered.
    """
    self._status = definitions.PROCESSING_STATUS_RUNNING
    self._number_of_consumed_errors = 0
    self._number_of_consumed_events = 0
    self._number_of_consumed_reports = 0
    self._number_of_consumed_sources = 0
    self._number_of_produced_errors = 0
    self._number_of_produced_events = 0
    self._number_of_produced_reports = 0
    self._number_of_produced_sources = 0

    number_of_filtered_events = 0

    logging.debug(u'Processing events.')

    filter_limit = getattr(event_filter, u'limit', None)

    for event in storage_writer.GetSortedEvents():
      if event_filter:
        filter_match = event_filter.Match(event)
      else:
        filter_match = None

      # pylint: disable=singleton-comparison
      if filter_match == False:
        number_of_filtered_events += 1
        continue

      for event_queue in self._event_queues.values():
        # TODO: Check for premature exit of analysis plugins.
        event_queue.PushItem(event)

      self._number_of_consumed_events += 1

      if (event_filter and filter_limit and
          filter_limit == self._number_of_consumed_events):
        break

    logging.debug(u'Finished pushing events to analysis plugins.')
    # Signal that we have finished adding events.
    for event_queue in self._event_queues.values():
      event_queue.PushItem(plaso_queue.QueueAbort(), block=False)

    logging.debug(u'Processing analysis plugin results.')

    # TODO: use a task based approach.
    plugin_names = [plugin_name for plugin_name in analysis_plugins.keys()]
    while plugin_names:
      for plugin_name in list(plugin_names):
        if self._abort:
          break

        # TODO: temporary solution.
        task = tasks.Task()
        task.identifier = plugin_name

        merge_ready = storage_writer.CheckTaskReadyForMerge(task)
        if merge_ready:
          self._status = definitions.PROCESSING_STATUS_MERGING

          event_queue = self._event_queues[plugin_name]
          del self._event_queues[plugin_name]

          event_queue.Close()

          storage_merge_reader = storage_writer.StartMergeTaskStorage(task)

          storage_merge_reader.MergeAttributeContainers()
          # TODO: temporary solution.
          plugin_names.remove(plugin_name)

          self._status = definitions.PROCESSING_STATUS_RUNNING

          self._number_of_produced_event_tags = (
              storage_writer.number_of_event_tags)
          self._number_of_produced_reports = (
              storage_writer.number_of_analysis_reports)

    try:
      storage_writer.StopTaskStorage(abort=self._abort)
    except (IOError, OSError) as exception:
      logging.error(u'Unable to stop task storage with error: {0:s}'.format(
          exception))

    if self._abort:
      logging.debug(u'Processing aborted.')
    else:
      logging.debug(u'Processing completed.')

    events_counter = collections.Counter()
    events_counter[u'Events filtered'] = number_of_filtered_events
    events_counter[u'Events processed'] = self._number_of_consumed_events

    return events_counter

  def _CheckStatusAnalysisProcess(self, pid):
    """Checks the status of an analysis process.

    Args:
      pid (int): process ID (PID) of a registered analysis process.

    Raises:
      KeyError: if the process is not registered with the engine.
    """
    # TODO: Refactor this method, simplify and separate concerns (monitoring
    # vs management).
    self._RaiseIfNotRegistered(pid)

    if pid in self._completed_analysis_processes:
      status_indicator = definitions.PROCESSING_STATUS_COMPLETED
      process_status = {
          u'processing_status': status_indicator}
      used_memory = 0

    else:
      process = self._processes_per_pid[pid]

      process_status = self._QueryProcessStatus(process)
      if process_status is None:
        process_is_alive = False
      else:
        process_is_alive = True

      process_information = self._process_information_per_pid[pid]
      used_memory = process_information.GetUsedMemory() or 0

      if used_memory > self._worker_memory_limit:
        logging.warning((
            u'Process: {0:s} (PID: {1:d}) killed because it exceeded the '
            u'memory limit: {2:d}.').format(
                process.name, pid, self._worker_memory_limit))
        self._KillProcess(pid)

      if isinstance(process_status, dict):
        self._rpc_errors_per_pid[pid] = 0
        status_indicator = process_status.get(u'processing_status', None)

        if status_indicator == definitions.PROCESSING_STATUS_COMPLETED:
          self._completed_analysis_processes.add(pid)

      else:
        rpc_errors = self._rpc_errors_per_pid.get(pid, 0) + 1
        self._rpc_errors_per_pid[pid] = rpc_errors

        if rpc_errors > self._MAXIMUM_RPC_ERRORS:
          process_is_alive = False

        if process_is_alive:
          rpc_port = process.rpc_port.value
          logging.warning((
              u'Unable to retrieve process: {0:s} (PID: {1:d}) status via '
              u'RPC socket: http://localhost:{2:d}').format(
                  process.name, pid, rpc_port))

          processing_status_string = u'RPC error'
          status_indicator = definitions.PROCESSING_STATUS_RUNNING
        else:
          processing_status_string = u'killed'
          status_indicator = definitions.PROCESSING_STATUS_KILLED

        process_status = {
            u'processing_status': processing_status_string}

    self._UpdateProcessingStatus(pid, process_status, used_memory)

    if status_indicator in definitions.PROCESSING_ERROR_STATUS:
      logging.error((
          u'Process {0:s} (PID: {1:d}) is not functioning correctly. '
          u'Status code: {2!s}.').format(
              process.name, pid, status_indicator))

      self._TerminateProcess(pid)

  def _ExportEvent(self, output_module, event, deduplicate_events=True):
    """Exports an event using an output module.

    Args:
      output_module (OutputModule): output module.
      event (EventObject): event.
      deduplicate_events (Optional[bool]): True if events should be
          deduplicated.
    """
    if event.timestamp != self._export_event_timestamp:
      self._FlushExportBuffer(
          output_module, deduplicate_events=deduplicate_events)
      self._export_event_timestamp = event.timestamp

    self._export_event_heap.PushEvent(event)

  def _ExportEvents(
      self, storage_reader, output_module, deduplicate_events=True,
      event_filter=None, time_slice=None, use_time_slicer=False):
    """Exports events using an output module.

    Args:
      storage_reader (StorageReader): storage reader.
      output_module (OutputModule): output module.
      deduplicate_events (Optional[bool]): True if events should be
          deduplicated.
      event_filter (Optional[FilterObject]): event filter.
      time_slice (Optional[TimeRange]): time range that defines a time slice
          to filter events.
      use_time_slicer (Optional[bool]): True if the 'time slicer' should be
          used. The 'time slicer' will provide a context of events around
          an event of interest.

    Returns:
      collections.Counter: counter that tracks the number of unique events
          read from storage.
    """
    self._status = definitions.PROCESSING_STATUS_EXPORTING

    time_slice_buffer = None
    if time_slice:
      if time_slice.event_timestamp is not None:
        time_slice = storage_time_range.TimeRange(
            time_slice.start_timestamp, time_slice.end_timestamp)

      if use_time_slicer:
        time_slice_buffer = bufferlib.CircularBuffer(time_slice.duration)

    filter_limit = getattr(event_filter, u'limit', None)
    forward_entries = 0

    number_of_filtered_events = 0
    number_of_events_from_time_slice = 0

    for event in storage_reader.GetSortedEvents(time_range=time_slice):
      if event_filter:
        filter_match = event_filter.Match(event)
      else:
        filter_match = None

      # pylint: disable=singleton-comparison
      if filter_match == False:
        if not time_slice_buffer:
          number_of_filtered_events += 1

        elif forward_entries == 0:
          time_slice_buffer.Append(event)
          number_of_filtered_events += 1

        elif forward_entries <= time_slice_buffer.size:
          self._ExportEvent(
              output_module, event, deduplicate_events=deduplicate_events)
          self._number_of_consumed_events += 1
          number_of_events_from_time_slice += 1
          forward_entries += 1

        else:
          # We reached the maximum size of the time slice and don't need to
          # include other entries.
          number_of_filtered_events += 1
          forward_entries = 0

      else:
        # pylint: disable=singleton-comparison
        if filter_match == True and time_slice_buffer:
          # Empty the time slice buffer.
          for event_in_buffer in time_slice_buffer.Flush():
            self._ExportEvent(
                output_module, event_in_buffer,
                deduplicate_events=deduplicate_events)
            self._number_of_consumed_events += 1
            number_of_filtered_events += 1
            number_of_events_from_time_slice += 1

          forward_entries = 1

        self._ExportEvent(
            output_module, event, deduplicate_events=deduplicate_events)
        self._number_of_consumed_events += 1

        # pylint: disable=singleton-comparison
        if (filter_match == True and filter_limit and
            filter_limit == self._number_of_consumed_events):
          break

    self._FlushExportBuffer(output_module)

    events_counter = collections.Counter()
    events_counter[u'Events filtered'] = number_of_filtered_events
    events_counter[u'Events from time slice'] = number_of_events_from_time_slice
    events_counter[u'Events processed'] = self._number_of_consumed_events

    if self._number_of_duplicate_events:
      events_counter[u'Duplicate events removed'] = (
          self._number_of_duplicate_events)

    if self._number_of_macb_grouped_events:
      events_counter[u'Events MACB grouped'] = (
          self._number_of_macb_grouped_events)

    if filter_limit:
      events_counter[u'Limited By'] = filter_limit

    return events_counter

  def _FlushExportBuffer(self, output_module, deduplicate_events=True):
    """Flushes buffered events and writes them to the output module.

    Args:
      output_module (OutputModule): output module.
      deduplicate_events (Optional[bool]): True if events should be
          deduplicated.
    """
    last_macb_group_identifier = None
    last_content_identifier = None
    macb_group = []

    generator = self._export_event_heap.PopEvents()

    for macb_group_identifier, content_identifier, event in generator:
      if deduplicate_events and last_content_identifier == content_identifier:
        self._number_of_duplicate_events += 1
        continue

      if macb_group_identifier is None:
        if macb_group:
          output_module.WriteEventMACBGroup(macb_group)
          macb_group = []

        output_module.WriteEvent(event)

      else:
        if (last_macb_group_identifier == macb_group_identifier or
            not macb_group):
          macb_group.append(event)

        else:
          output_module.WriteEventMACBGroup(macb_group)
          macb_group = [event]

        self._number_of_macb_grouped_events += 1

      last_macb_group_identifier = macb_group_identifier
      last_content_identifier = content_identifier

    if macb_group:
      output_module.WriteEventMACBGroup(macb_group)

  def _StartAnalysisProcesses(self, storage_writer, analysis_plugins):
    """Starts the analysis processes.

    Args:
      storage_writer (StorageWriter): storage writer.
      analysis_plugins (dict[str, AnalysisPlugin]): analysis plugins that
          should be run and their names.
    """
    logging.info(u'Starting analysis plugins.')

    for analysis_plugin in analysis_plugins.values():
      self._analysis_plugins[analysis_plugin.NAME] = analysis_plugin

      process = self._StartWorkerProcess(analysis_plugin.NAME, storage_writer)
      if not process:
        logging.error(u'Unable to create analysis process: {0:s}'.format(
            analysis_plugin.NAME))

    logging.info(u'Analysis plugins running')

  def _StatusUpdateThreadMain(self):
    """Main function of the status update thread."""
    while self._status_update_active:
      # Make a local copy of the PIDs in case the dict is changed by
      # the main thread.
      for pid in list(self._process_information_per_pid.keys()):
        self._CheckStatusAnalysisProcess(pid)

      used_memory = self._process_information.GetUsedMemory() or 0

      display_name = getattr(self._merge_task, u'identifier', u'')

      self._processing_status.UpdateForemanStatus(
          self._name, self._status, self._pid, used_memory, display_name,
          self._number_of_consumed_sources, self._number_of_produced_sources,
          self._number_of_consumed_events, self._number_of_produced_events,
          self._number_of_consumed_event_tags,
          self._number_of_produced_event_tags,
          self._number_of_consumed_errors, self._number_of_produced_errors,
          self._number_of_consumed_reports, self._number_of_produced_reports)

      if self._status_update_callback:
        self._status_update_callback(self._processing_status)

      time.sleep(self._STATUS_UPDATE_INTERVAL)

  def _StopAnalysisProcesses(self, abort=False):
    """Stops the analysis processes.

    Args:
      abort (bool): True to indicated the stop is issued on abort.
    """
    logging.debug(u'Stopping analysis processes.')
    self._StopMonitoringProcesses()

    # Note that multiprocessing.Queue is very sensitive regarding
    # blocking on either a get or a put. So we try to prevent using
    # any blocking behavior.

    if abort:
      # Signal all the processes to abort.
      self._AbortTerminate()

    if not self._use_zeromq:
      logging.debug(u'Emptying queues.')
      for event_queue in self._event_queues.values():
        event_queue.Empty()

    # Wake the processes to make sure that they are not blocking
    # waiting for the queue new items.
    for event_queue in self._event_queues.values():
      event_queue.PushItem(plaso_queue.QueueAbort(), block=False)

    # Try waiting for the processes to exit normally.
    self._AbortJoin(timeout=self._PROCESS_JOIN_TIMEOUT)
    for event_queue in self._event_queues.values():
      event_queue.Close(abort=abort)

    if abort:
      # Kill any remaining processes.
      self._AbortKill()
    else:
      # Check if the processes are still alive and terminate them if necessary.
      self._AbortTerminate()
      self._AbortJoin(timeout=self._PROCESS_JOIN_TIMEOUT)

      for event_queue in self._event_queues.values():
        event_queue.Close(abort=True)

  def _UpdateProcessingStatus(self, pid, process_status, used_memory):
    """Updates the processing status.

    Args:
      pid (int): process identifier (PID) of the worker process.
      process_status (dict[str, object]): status values received from
          the worker process.
      used_memory (int): size of used memory in bytes.

    Raises:
      KeyError: if the process is not registered with the engine.
    """
    self._RaiseIfNotRegistered(pid)

    if not process_status:
      return

    process = self._processes_per_pid[pid]

    processing_status = process_status.get(u'processing_status', None)

    self._RaiseIfNotMonitored(pid)

    display_name = process_status.get(u'display_name', u'')
    number_of_consumed_errors = process_status.get(
        u'number_of_consumed_errors', None)
    number_of_produced_errors = process_status.get(
        u'number_of_produced_errors', None)

    number_of_consumed_event_tags = process_status.get(
        u'number_of_consumed_event_tags', None)
    number_of_produced_event_tags = process_status.get(
        u'number_of_produced_event_tags', None)

    number_of_consumed_events = process_status.get(
        u'number_of_consumed_events', None)
    number_of_produced_events = process_status.get(
        u'number_of_produced_events', None)

    number_of_consumed_reports = process_status.get(
        u'number_of_consumed_reports', None)
    number_of_produced_reports = process_status.get(
        u'number_of_produced_reports', None)

    number_of_consumed_sources = process_status.get(
        u'number_of_consumed_sources', None)
    number_of_produced_sources = process_status.get(
        u'number_of_produced_sources', None)

    if processing_status != definitions.PROCESSING_STATUS_IDLE:
      last_activity_timestamp = process_status.get(
          u'last_activity_timestamp', 0.0)

      if last_activity_timestamp:
        last_activity_timestamp += self._PROCESS_WORKER_TIMEOUT

        current_timestamp = time.time()
        if current_timestamp > last_activity_timestamp:
          logging.error((
              u'Process {0:s} (PID: {1:d}) has not reported activity within '
              u'the timeout period.').format(process.name, pid))
          processing_status = definitions.PROCESSING_STATUS_NOT_RESPONDING

    self._processing_status.UpdateWorkerStatus(
        process.name, processing_status, pid, used_memory, display_name,
        number_of_consumed_sources, number_of_produced_sources,
        number_of_consumed_events, number_of_produced_events,
        number_of_consumed_event_tags, number_of_produced_event_tags,
        number_of_consumed_errors, number_of_produced_errors,
        number_of_consumed_reports, number_of_produced_reports)

  def _StartWorkerProcess(self, process_name, storage_writer):
    """Creates, starts, monitors and registers a worker process.

    Args:
      process_name (str): process name.
      storage_writer (StorageWriter): storage writer for a session storage used
          to create task storage.

    Returns:
      MultiProcessWorkerProcess: extraction worker process.
    """
    analysis_plugin = self._analysis_plugins.get(process_name, None)
    if not analysis_plugin:
      logging.error(u'Missing analysis plugin: {0:s}'.format(process_name))
      return

    if self._use_zeromq:
      queue_name = u'{0:s} output event queue'.format(process_name)
      output_event_queue = zeromq_queue.ZeroMQPushBindQueue(
          name=queue_name, timeout_seconds=self._QUEUE_TIMEOUT)
      # Open the queue so it can bind to a random port, and we can get the
      # port number to use in the input queue.
      output_event_queue.Open()

    else:
      output_event_queue = multi_process_queue.MultiProcessingQueue(
          timeout=self._QUEUE_TIMEOUT)

    self._event_queues[process_name] = output_event_queue

    if self._use_zeromq:
      queue_name = u'{0:s} input event queue'.format(process_name)
      input_event_queue = zeromq_queue.ZeroMQPullConnectQueue(
          name=queue_name, delay_open=True, port=output_event_queue.port,
          timeout_seconds=self._QUEUE_TIMEOUT)

    else:
      input_event_queue = output_event_queue

    process = analysis_process.AnalysisProcess(
        input_event_queue, storage_writer, self._knowledge_base,
        analysis_plugin, data_location=self._data_location,
        event_filter_expression=self._event_filter_expression,
        name=process_name)

    process.start()

    logging.info(u'Started analysis plugin: {0:s} (PID: {1:d}).'.format(
        process_name, process.pid))

    try:
      self._StartMonitoringProcess(process)
    except (IOError, KeyError) as exception:
      logging.error((
          u'Unable to monitor analysis plugin: {0:s} (PID: {1:d}) '
          u'with error: {2!s}').format(process_name, process.pid, exception))

      process.terminate()
      return

    self._RegisterProcess(process)
    return process

  def AnalyzeEvents(
      self, knowledge_base_object, storage_writer, data_location,
      analysis_plugins, event_filter=None, event_filter_expression=None,
      status_update_callback=None, worker_memory_limit=None):
    """Analyzes events in a plaso storage.

    Args:
      knowledge_base_object (KnowledgeBase): contains information from
          the source data needed for processing.
      storage_writer (StorageWriter): storage writer.
      data_location (str): path to the location that data files should
          be loaded from.
      analysis_plugins (dict[str, AnalysisPlugin]): analysis plugins that
          should be run and their names.
      event_filter (Optional[FilterObject]): event filter.
      event_filter_expression (Optional[str]): event filter expression.
      status_update_callback (Optional[function]): callback function for status
          updates.
      worker_memory_limit (Optional[int]): maximum amount of memory a worker is
          allowed to consume, where None represents the default memory limit.

    Raises:
      KeyboardInterrupt: if a keyboard interrupt was raised.
    """
    if not analysis_plugins:
      return

    keyboard_interrupt = False

    self._analysis_plugins = {}
    self._data_location = data_location
    self._event_filter_expression = event_filter_expression
    self._knowledge_base = knowledge_base_object
    self._status_update_callback = status_update_callback
    self._worker_memory_limit = (
        worker_memory_limit or definitions.DEFAULT_WORKER_MEMORY_LIMIT)

    # Set up the storage writer before the analysis processes.
    storage_writer.StartTaskStorage()

    self._StartAnalysisProcesses(storage_writer, analysis_plugins)

    # Start the status update thread after open of the storage writer
    # so we don't have to clean up the thread if the open fails.
    self._StartStatusUpdateThread()

    try:
      # Open the storage file after creating the worker processes otherwise
      # the ZIP storage file will remain locked as long as the worker processes
      # are alive.
      storage_writer.Open()
      storage_writer.ReadPreprocessingInformation(knowledge_base_object)
      storage_writer.WriteSessionStart()

      try:
        self._AnalyzeEvents(
            storage_writer, analysis_plugins, event_filter=event_filter)

        self._status = definitions.PROCESSING_STATUS_FINALIZING

      except KeyboardInterrupt:
        keyboard_interrupt = True
        self._abort = True

        self._processing_status.aborted = True
        if self._status_update_callback:
          self._status_update_callback(self._processing_status)

      finally:
        storage_writer.WriteSessionCompletion(aborted=self._abort)

        storage_writer.Close()

    finally:
      # Stop the status update thread after close of the storage writer
      # so we include the storage sync to disk in the status updates.
      self._StopStatusUpdateThread()

    try:
      self._StopAnalysisProcesses(abort=self._abort)

    except KeyboardInterrupt:
      keyboard_interrupt = True

      self._AbortKill()

      # The abort can leave the main process unresponsive
      # due to incorrectly finalized IPC.
      self._KillProcess(os.getpid())

    # Reset values.
    self._analysis_plugins = {}
    self._data_location = None
    self._event_filter_expression = None
    self._knowledge_base = None
    self._status_update_callback = None
    self._worker_memory_limit = definitions.DEFAULT_WORKER_MEMORY_LIMIT

    if keyboard_interrupt:
      raise KeyboardInterrupt

    if keyboard_interrupt:
      raise KeyboardInterrupt

  def ExportEvents(
      self, knowledge_base_object, storage_reader, output_module,
      deduplicate_events=True, event_filter=None, status_update_callback=None,
      time_slice=None, use_time_slicer=False):
    """Exports events using an output module.

    Args:
      knowledge_base_object (KnowledgeBase): contains information from
          the source data needed for processing.
      storage_reader (StorageReader): storage reader.
      output_module (OutputModule): output module.
      deduplicate_events (Optional[bool]): True if events should be
          deduplicated.
      event_filter (Optional[FilterObject]): event filter.
      status_update_callback (Optional[function]): callback function for status
          updates.
      time_slice (Optional[TimeSlice]): slice of time to output.
      use_time_slicer (Optional[bool]): True if the 'time slicer' should be
          used. The 'time slicer' will provide a context of events around
          an event of interest.

    Returns:
      collections.Counter: counter that tracks the number of events extracted
          from storage.
    """
    self._status_update_callback = status_update_callback

    storage_reader.ReadPreprocessingInformation(knowledge_base_object)

    output_module.Open()
    output_module.WriteHeader()

    self._StartStatusUpdateThread()

    try:
      events_counter = self._ExportEvents(
          storage_reader, output_module, deduplicate_events=deduplicate_events,
          event_filter=event_filter, time_slice=time_slice,
          use_time_slicer=use_time_slicer)

    finally:
      # Stop the status update thread after close of the storage writer
      # so we include the storage sync to disk in the status updates.
      self._StopStatusUpdateThread()

    output_module.WriteFooter()
    output_module.Close()

    # Reset values.
    self._status_update_callback = None

    return events_counter



# -*- coding: utf-8 -*-
"""The Microsoft IIS log file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


__author__ = 'Ashley Holtz (ashley.a.holtz@gmail.com)'


class IISLogFileEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Microsoft IIS log file event."""

  DATA_TYPE = u'iis:log:line'

  FORMAT_STRING_PIECES = [
      u'{http_method}',
      u'{requested_uri_stem}',
      u'[',
      u'{source_ip}',
      u'>',
      u'{dest_ip}',
      u':',
      u'{dest_port}',
      u']',
      u'HTTP Status: {http_status}',
      u'Bytes Sent: {sent_bytes}',
      u'Bytes Received: {received_bytes}',
      u'User Agent: {user_agent}',
      u'Protocol Version: {protocol_version}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{http_method}',
      u'{requested_uri_stem}',
      u'[',
      u'{source_ip}',
      u'>',
      u'{dest_ip}',
      u':',
      u'{dest_port}',
      u']']

  SOURCE_LONG = u'IIS Log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(IISLogFileEventFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the parsers CLI arguments helper."""

import argparse
import unittest

from plaso.cli import tools
from plaso.cli.helpers import parsers
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class ParsersArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the parsers CLI arguments helper."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--parsers PARSER_LIST]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --parsers PARSER_LIST',
      (u'                        Define a list of parsers to use by the tool. '
       u'This is a'),
      (u'                        comma separated list where each entry can be '
       u'either a'),
      (u'                        name of a parser or a parser list. Each entry '
       u'can be'),
      (u'                        prepended with an exclamation mark to negate '
       u'the'),
      (u'                        selection (exclude it). The list match is an '
       u'exact'),
      (u'                        match while an individual parser matching is '
       u'a case'),
      (u'                        insensitive substring match, with support for '
       u'glob'),
      (u'                        patterns. Examples would be: "reg" that '
       u'matches the'),
      (u'                        substring "reg" in all parser names or the '
       u'glob'),
      (u'                        pattern "sky[pd]" that would match all '
       u'parsers that'),
      (u'                        have the string "skyp" or "skyd" in its '
       u'name. All'),
      (u'                        matching is case insensitive. Use "--parsers '
       u'list" or'),
      u'                        "--info" to list the available parsers.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py', description=u'Test argument parser.',
        add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    parsers.ParsersArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.parsers = u'winevt'

    test_tool = tools.CLITool()
    parsers.ParsersArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._parser_filter_expression, options.parsers)

    with self.assertRaises(errors.BadConfigObject):
      parsers.ParsersArgumentsHelper.ParseOptions(options, None)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The language CLI arguments helper."""

from plaso.cli import tools
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.lib import errors


class LanguageArgumentsHelper(interface.ArgumentsHelper):
  """Language CLI arguments helper."""

  NAME = u'language'
  DESCRIPTION = u'Language command line arguments.'

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        u'--language', metavar=u'LANGUAGE', dest=u'preferred_language',
        default=u'en-US', type=str, help=(
            u'The preferred language identifier for Windows Event Log message '
            u'strings. Use "--language list" to see a list of available '
            u'language identifiers. Note that formatting will fall back on '
            u'en-US (LCID 0x0409) if the preferred language is not available '
            u'in the database of message string templates.'))

  @classmethod
  def ParseOptions(cls, options, configuration_object):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      configuration_object (CLITool): object to be configured by the argument
          helper.

    Raises:
      BadConfigObject: when the configuration object is of the wrong type.
    """
    if not isinstance(configuration_object, tools.CLITool):
      raise errors.BadConfigObject(
          u'Configuration object is not an instance of CLITool')

    preferred_language = cls._ParseStringOption(
        options, u'preferred_language', default_value=u'en-US')

    setattr(configuration_object, u'_preferred_language', preferred_language)


manager.ArgumentHelperManager.RegisterHelper(LanguageArgumentsHelper)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This file contains the tests for the generic text parser."""

import unittest

import pyparsing

from plaso.parsers import text_parser

from tests.parsers import test_lib


class PyparsingConstantsTest(test_lib.ParserTestCase):
  """Tests the PyparsingConstants text parser."""

  def testConstants(self):
    """Tests parsing with constants."""
    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.MONTH.parseString(u'MMo')
    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.MONTH.parseString(u'M')
    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.MONTH.parseString(u'March', parseAll=True)

    self.assertTrue(text_parser.PyparsingConstants.MONTH.parseString(u'Jan'))

    line = u'# This is a comment.'
    parsed_line = text_parser.PyparsingConstants.COMMENT_LINE_HASH.parseString(
        line)
    self.assertEqual(parsed_line[-1], u'This is a comment.')
    self.assertEqual(len(parsed_line), 2)

  def testConstantIPv4(self):
    """Tests parsing with the IPV4_ADDRESS constant."""
    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(
            u'123.51.234.52'))
    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(
            u'255.254.23.1'))
    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'1.1.34.2'))

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'1.1.34.258')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'a.1.34.258')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'.34.258')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'34.258')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_ADDRESS.parseString(u'10.52.34.258')

  def testConstantOctet(self):
    """Tests parsing with the IPV4_OCTET constant."""
    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_OCTET.parseString(u'0'))

    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_OCTET.parseString(u'123'))

    self.assertTrue(
        text_parser.PyparsingConstants.IPV4_OCTET.parseString(u'255'))

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_OCTET.parseString(u'526')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_OCTET.parseString(u'1026')

    with self.assertRaises(pyparsing.ParseException):
      text_parser.PyparsingConstants.IPV4_OCTET.parseString(
          u'a9', parseAll=True)


if __name__ == u'__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a parser for the Android contacts2 Call History.

Android Call History is stored in SQLite database files named contacts2.db.
"""

from dfdatetime import java_time as dfdatetime_java_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import py2to3
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class AndroidCallEventData(events.EventData):
  """Android Call event data.

  Attributes:
    call_type (str): type of call, such as: Incoming, Outgoing, or Missed.
    duration (int): number of seconds the call lasted.
    name (str): name associated to the remote party.
    number (str): phone number associated to the remote party.
  """

  DATA_TYPE = u'android:event:call'

  def __init__(self):
    """Initializes event data."""
    super(AndroidCallEventData, self).__init__(data_type=self.DATA_TYPE)
    self.call_type = None
    self.duration = None
    self.name = None
    self.number = None


class AndroidCallPlugin(interface.SQLitePlugin):
  """Parse Android contacts2 database."""

  NAME = u'android_calls'
  DESCRIPTION = u'Parser for Android calls SQLite database files.'

  REQUIRED_TABLES = frozenset([u'calls'])

  # Define the needed queries.
  QUERIES = [
      (u'SELECT _id AS id, date, number, name, duration, type FROM calls',
       u'ParseCallsRow')]

  SCHEMAS = [{
      u'_sync_state': (
          u'CREATE TABLE _sync_state (_id INTEGER PRIMARY KEY, account_name '
          u'TEXT NOT NULL, account_type TEXT NOT NULL, data TEXT, '
          u'UNIQUE(account_name, account_type))'),
      u'_sync_state_metadata': (
          u'CREATE TABLE _sync_state_metadata (version INTEGER)'),
      u'accounts': (
          u'CREATE TABLE accounts (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'account_name TEXT, account_type TEXT, data_set TEXT)'),
      u'agg_exceptions': (
          u'CREATE TABLE agg_exceptions (_id INTEGER PRIMARY KEY '
          u'AUTOINCREMENT, type INTEGER NOT NULL, raw_contact_id1 INTEGER '
          u'REFERENCES raw_contacts(_id), raw_contact_id2 INTEGER REFERENCES '
          u'raw_contacts(_id))'),
      u'android_metadata': (
          u'CREATE TABLE android_metadata (locale TEXT)'),
      u'calls': (
          u'CREATE TABLE calls (_id INTEGER PRIMARY KEY AUTOINCREMENT, number '
          u'TEXT, date INTEGER, duration INTEGER, type INTEGER, new INTEGER, '
          u'name TEXT, numbertype INTEGER, numberlabel TEXT, countryiso TEXT, '
          u'voicemail_uri TEXT, is_read INTEGER, geocoded_location TEXT, '
          u'lookup_uri TEXT, matched_number TEXT, normalized_number TEXT, '
          u'photo_id INTEGER NOT NULL DEFAULT 0, formatted_number TEXT, _data '
          u'TEXT, has_content INTEGER, mime_type TEXT, source_data TEXT, '
          u'source_package TEXT, state INTEGER)'),
      u'contacts': (
          u'CREATE TABLE contacts (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'name_raw_contact_id INTEGER REFERENCES raw_contacts(_id), photo_id '
          u'INTEGER REFERENCES data(_id), photo_file_id INTEGER REFERENCES '
          u'photo_files(_id), custom_ringtone TEXT, send_to_voicemail INTEGER '
          u'NOT NULL DEFAULT 0, times_contacted INTEGER NOT NULL DEFAULT 0, '
          u'last_time_contacted INTEGER, starred INTEGER NOT NULL DEFAULT 0, '
          u'has_phone_number INTEGER NOT NULL DEFAULT 0, lookup TEXT, '
          u'status_update_id INTEGER REFERENCES data(_id), '
          u'contact_last_updated_timestamp INTEGER)'),
      u'data': (
          u'CREATE TABLE data (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'package_id INTEGER REFERENCES package(_id), mimetype_id INTEGER '
          u'REFERENCES mimetype(_id) NOT NULL, raw_contact_id INTEGER '
          u'REFERENCES raw_contacts(_id) NOT NULL, is_read_only INTEGER NOT '
          u'NULL DEFAULT 0, is_primary INTEGER NOT NULL DEFAULT 0, '
          u'is_super_primary INTEGER NOT NULL DEFAULT 0, data_version INTEGER '
          u'NOT NULL DEFAULT 0, data1 TEXT, data2 TEXT, data3 TEXT, data4 '
          u'TEXT, data5 TEXT, data6 TEXT, data7 TEXT, data8 TEXT, data9 TEXT, '
          u'data10 TEXT, data11 TEXT, data12 TEXT, data13 TEXT, data14 TEXT, '
          u'data15 TEXT, data_sync1 TEXT, data_sync2 TEXT, data_sync3 TEXT, '
          u'data_sync4 TEXT )'),
      u'data_usage_stat': (
          u'CREATE TABLE data_usage_stat(stat_id INTEGER PRIMARY KEY '
          u'AUTOINCREMENT, data_id INTEGER NOT NULL, usage_type INTEGER NOT '
          u'NULL DEFAULT 0, times_used INTEGER NOT NULL DEFAULT 0, '
          u'last_time_used INTEGER NOT NULL DEFAULT 0, FOREIGN KEY(data_id) '
          u'REFERENCES data(_id))'),
      u'default_directory': (
          u'CREATE TABLE default_directory (_id INTEGER PRIMARY KEY)'),
      u'deleted_contacts': (
          u'CREATE TABLE deleted_contacts (contact_id INTEGER PRIMARY KEY, '
          u'contact_deleted_timestamp INTEGER NOT NULL default 0)'),
      u'directories': (
          u'CREATE TABLE directories(_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'packageName TEXT NOT NULL, authority TEXT NOT NULL, typeResourceId '
          u'INTEGER, typeResourceName TEXT, accountType TEXT, accountName '
          u'TEXT, displayName TEXT, exportSupport INTEGER NOT NULL DEFAULT 0, '
          u'shortcutSupport INTEGER NOT NULL DEFAULT 0, photoSupport INTEGER '
          u'NOT NULL DEFAULT 0)'),
      u'groups': (
          u'CREATE TABLE groups (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'package_id INTEGER REFERENCES package(_id), account_name STRING '
          u'DEFAULT NULL, account_type STRING DEFAULT NULL, data_set STRING '
          u'DEFAULT NULL, sourceid TEXT, version INTEGER NOT NULL DEFAULT 1, '
          u'dirty INTEGER NOT NULL DEFAULT 0, title TEXT, title_res INTEGER, '
          u'notes TEXT, system_id TEXT, deleted INTEGER NOT NULL DEFAULT 0, '
          u'group_visible INTEGER NOT NULL DEFAULT 0, should_sync INTEGER NOT '
          u'NULL DEFAULT 1, auto_add INTEGER NOT NULL DEFAULT 0, favorites '
          u'INTEGER NOT NULL DEFAULT 0, group_is_read_only INTEGER NOT NULL '
          u'DEFAULT 0, sync1 TEXT, sync2 TEXT, sync3 TEXT, sync4 TEXT , '
          u'account_id INTEGER REFERENCES accounts(_id))'),
      u'mimetypes': (
          u'CREATE TABLE mimetypes (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'mimetype TEXT NOT NULL)'),
      u'name_lookup': (
          u'CREATE TABLE name_lookup (data_id INTEGER REFERENCES data(_id) NOT '
          u'NULL, raw_contact_id INTEGER REFERENCES raw_contacts(_id) NOT '
          u'NULL, normalized_name TEXT NOT NULL, name_type INTEGER NOT NULL, '
          u'PRIMARY KEY (data_id, normalized_name, name_type))'),
      u'nickname_lookup': (
          u'CREATE TABLE nickname_lookup (name TEXT, cluster TEXT)'),
      u'packages': (
          u'CREATE TABLE packages (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'package TEXT NOT NULL)'),
      u'phone_lookup': (
          u'CREATE TABLE phone_lookup (data_id INTEGER REFERENCES data(_id) '
          u'NOT NULL, raw_contact_id INTEGER REFERENCES raw_contacts(_id) NOT '
          u'NULL, normalized_number TEXT NOT NULL, min_match TEXT NOT NULL)'),
      u'photo_files': (
          u'CREATE TABLE photo_files (_id INTEGER PRIMARY KEY AUTOINCREMENT, '
          u'height INTEGER NOT NULL, width INTEGER NOT NULL, filesize INTEGER '
          u'NOT NULL)'),
      u'properties': (
          u'CREATE TABLE properties (property_key TEXT PRIMARY KEY, '
          u'property_value TEXT )')}]

  CALL_TYPE = {
      1: u'INCOMING',
      2: u'OUTGOING',
      3: u'MISSED'}

  def ParseCallsRow(self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a Call record row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    call_type = self.CALL_TYPE.get(row['type'], u'UNKNOWN')
    duration = row['duration']
    timestamp = row['date']

    event_data = AndroidCallEventData()
    event_data.call_type = call_type
    event_data.duration = row['duration']
    event_data.name = row['name']
    event_data.number = row['number']
    event_data.offset = row['id']
    event_data.query = query

    date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(date_time, u'Call Started')
    parser_mediator.ProduceEventWithEventData(event, event_data)

    if duration:
      if isinstance(duration, py2to3.STRING_TYPES):
        try:
          duration = int(duration, 10)
        except ValueError:
          duration = 0

      # The duration is in seconds and the date value in milliseconds.
      timestamp += duration * 1000

      date_time = dfdatetime_java_time.JavaTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(date_time, u'Call Ended')
      parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(AndroidCallPlugin)



# -*- coding: utf-8 -*-
"""XML RPC proxy server and client."""

import logging
import sys
import threading

if sys.version_info[0] < 3:
  import xmlrpclib
  import SimpleXMLRPCServer
  import SocketServer
else:
  from xmlrpc.server import SimpleXMLRPCServer  # pylint: disable=import-error
  from xmlrpc import client as xmlrpclib  # pylint: disable=import-error
  import socketserver as SocketServer  # pylint: disable=import-error

from xml.parsers import expat # pylint: disable=wrong-import-order

from plaso.multi_processing import rpc


class XMLRPCClient(rpc.RPCClient):
  """Class that defines the XML RPC client."""

  _RPC_FUNCTION_NAME = u''

  def __init__(self):
    """Initializes the process status RPC client object."""
    super(XMLRPCClient, self).__init__()
    self._xmlrpc_proxy = None

  def CallFunction(self):
    """Calls the function via RPC."""
    if self._xmlrpc_proxy is None:
      return

    rpc_call = getattr(self._xmlrpc_proxy, self._RPC_FUNCTION_NAME, None)
    if rpc_call is None:
      return

    try:
      return rpc_call()
    except (
        expat.ExpatError, SocketServer.socket.error,
        xmlrpclib.Fault) as exception:
      logging.warning(u'Error while making RPC call: {0:s}'.format(exception))
      return

  def Close(self):
    """Closes the RPC communication channel to the server."""
    self._xmlrpc_proxy = None

  def Open(self, hostname, port):
    """Opens a RPC communication channel to the server.

    Args:
      hostname: the hostname or IP address to connect to for requests.
      port: the port to connect to for requests.

    Returns:
      A boolean indicating if the communication channel was established.
    """
    server_url = u'http://{0:s}:{1:d}'.format(hostname, port)
    try:
      self._xmlrpc_proxy = xmlrpclib.ServerProxy(server_url, allow_none=True)
    except SocketServer.socket.error as exception:
      logging.warning((
          u'Unable to connect to RPC server on {0:s}:{1:d} with error: '
          u'{2:s}').format(hostname, port, exception))
      return False

    return True


class ThreadedXMLRPCServer(rpc.RPCServer):
  """Class that defines the threaded XML RPC server."""

  _RPC_FUNCTION_NAME = u''
  _THREAD_NAME = u''

  def __init__(self, callback):
    """Initialize the RPC server.

    Args:
      callback: the callback function to invoke on get status RPC request.
    """
    super(ThreadedXMLRPCServer, self).__init__(callback)
    self._rpc_thread = None
    self._xmlrpc_server = None

  def _Close(self):
    """Closes the RPC communication channel for clients."""
    if not self._xmlrpc_server:
      return
    self._xmlrpc_server.shutdown()
    self._xmlrpc_server = None

  def _Open(self, hostname, port):
    """Opens the RPC communication channel for clients.

    Args:
      hostname: the hostname or IP address to connect to for requests.
      port: the port to connect to for requests.

    Returns:
      A boolean indicating if the communication channel was successfully opened.
    """
    try:
      self._xmlrpc_server = SimpleXMLRPCServer.SimpleXMLRPCServer(
          (hostname, port), logRequests=False, allow_none=True)
    except SocketServer.socket.error as exception:
      logging.warning((
          u'Unable to bind a RPC server on {0:s}:{1:d} with error: '
          u'{2:s}').format(hostname, port, exception))
      return False

    self._xmlrpc_server.register_function(
        self._callback, self._RPC_FUNCTION_NAME)
    return True

  def Start(self, hostname, port):
    """Starts the process status RPC server.

    Args:
      hostname: the hostname or IP address to connect to for requests.
      port: the port to connect to for requests.

    Returns:
      A boolean indicating if the RPC server was successfully started.
    """
    if not self._Open(hostname, port):
      return False

    self._rpc_thread = threading.Thread(
        name=self._THREAD_NAME, target=self._xmlrpc_server.serve_forever)
    self._rpc_thread.start()
    return True

  def Stop(self):
    """Stops the process status RPC server."""
    self._Close()

    if self._rpc_thread.isAlive():
      self._rpc_thread.join()
    self._rpc_thread = None


class XMLProcessStatusRPCClient(XMLRPCClient):
  """Class that defines a XML process status RPC client."""

  _RPC_FUNCTION_NAME = u'status'


class XMLProcessStatusRPCServer(ThreadedXMLRPCServer):
  """Class that defines a XML process status RPC server."""

  _RPC_FUNCTION_NAME = u'status'
  _THREAD_NAME = u'process_status_rpc_server'



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the SQLite plugin interface."""

import sys
import unittest

from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface

from tests import test_lib as shared_test_lib
from tests.parsers.sqlite_plugins import test_lib


class TestSQLitePlugin(interface.SQLitePlugin):
  """Convenience class for a test SQLite plugin."""

  NAME = u'test'

  QUERIES = [(
      u'SELECT Field1, Field2, Field3 FROM MyTable', u'ParseMyTableRow')]

  REQUIRED_TABLES = frozenset([u'MyTable'])

  SCHEMAS = [
      {u'MyTable':
       u'CREATE TABLE "MyTable" ( `Field1` TEXT, `Field2` INTEGER, '
       u'`Field3` BLOB )'}]

  def __init__(self):
    """Initializes SQLite plugin."""
    super(TestSQLitePlugin, self).__init__()
    self.results = []

  def ParseMyTableRow(self, parser_mediator, row, **unused_kwargs):
    """Parses a MyTable row.

    Args:
      parser_mediator: A parser mediator object (instance of ParserMediator).
      row: The row resulting from the query.
    """
    file_entry = parser_mediator.GetFileEntry()
    path_spec = file_entry.path_spec
    location = path_spec.location
    from_wal = location.endswith(u'-wal')
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".
    # Also, Field3 needs to be converted to a string if Python 2 is used
    # because it is a read-write buffer.
    field3 = row['Field3']
    if sys.version_info[0] < 3:
      field3 = str(field3)
    self.results.append(((row['Field1'], row['Field2'], field3), from_wal))

    event = time_events.TimestampEvent(
        timelib.Timestamp.NONE_TIMESTAMP,
        definitions.TIME_DESCRIPTION_NOT_A_TIME, data_type=u'fake')
    event.field1 = row['Field1']
    event.field2 = row['Field2']
    event.field3 = field3
    event.from_wal = location.endswith(u'-wal')
    parser_mediator.ProduceEvent(event)


class SQLiteInterfaceTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite plugin interface."""

  @shared_test_lib.skipUnlessHasTestFile([u'wal_database.db'])
  @shared_test_lib.skipUnlessHasTestFile([u'wal_database.db-wal'])
  def testProcessWithWAL(self):
    """Tests the Process function on a database with WAL file."""
    plugin = TestSQLitePlugin()
    cache = sqlite.SQLiteCache()
    wal_file = self._GetTestFilePath([u'wal_database.db-wal'])
    self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache, wal_path=wal_file)

    expected_results = [
        ((u'Committed Text 1', 1, b'None'), False),
        ((u'Committed Text 2', 2, b'None'), False),
        ((u'Deleted Text 1', 3, b'None'), False),
        ((u'Committed Text 3', 4, b'None'), False),
        ((u'Committed Text 4', 5, b'None'), False),
        ((u'Deleted Text 2', 6, b'None'), False),
        ((u'Committed Text 5', 7, b'None'), False),
        ((u'Committed Text 6', 8, b'None'), False),
        ((u'Committed Text 7', 9, b'None'), False),
        ((u'Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03'), False),
        ((u'Modified Committed Text 3', 4, b'None'), True),
        ((u'Unhashable Row 2', 11, b'More Binary Text!\x01\x02\x03'), True),
        ((u'New Text 1', 12, b'None'), True),
        ((u'New Text 2', 13, b'None'), True)]

    self.assertEqual(expected_results, plugin.results)

  @shared_test_lib.skipUnlessHasTestFile([u'wal_database.db'])
  def testProcessWithoutWAL(self):
    """Tests the Process function on a database without WAL file."""
    plugin = TestSQLitePlugin()
    cache = sqlite.SQLiteCache()
    self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache)

    expected_results = [
        ((u'Committed Text 1', 1, b'None'), False),
        ((u'Committed Text 2', 2, b'None'), False),
        ((u'Deleted Text 1', 3, b'None'), False),
        ((u'Committed Text 3', 4, b'None'), False),
        ((u'Committed Text 4', 5, b'None'), False),
        ((u'Deleted Text 2', 6, b'None'), False),
        ((u'Committed Text 5', 7, b'None'), False),
        ((u'Committed Text 6', 8, b'None'), False),
        ((u'Committed Text 7', 9, b'None'), False),
        ((u'Unhashable Row 1', 10, b'Binary Text!\x01\x02\x03'), False)]

    self.assertEqual(expected_results, plugin.results)

  @shared_test_lib.skipUnlessHasTestFile([u'wal_database.db'])
  @shared_test_lib.skipUnlessHasTestFile([u'wal_database.db-wal'])
  def testSchemaMatching(self):
    """Tests the Schema matching capabilities."""
    plugin = TestSQLitePlugin()
    cache = sqlite.SQLiteCache()

    # Test matching schema.
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache)
    for event in storage_writer.GetEvents():
      self.assertTrue(event.schema_match)

    # Test schema change with WAL.
    wal_file = self._GetTestFilePath([u'wal_database.db-wal'])
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache, wal_path=wal_file)

    for event in storage_writer.GetEvents():
      if event.from_wal:
        self.assertFalse(event.schema_match)
      else:
        self.assertTrue(event.schema_match)

    # Add schema change from WAL file and test again.
    plugin.SCHEMAS.append(
        {u'MyTable':
         u'CREATE TABLE "MyTable" ( `Field1` TEXT, `Field2` INTEGER, `Field3` '
         u'BLOB , NewField TEXT)',
         u'NewTable':
         u'CREATE TABLE NewTable(NewTableField1 TEXT, NewTableField2 TEXT)'})

    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache, wal_path=wal_file)
    for event in storage_writer.GetEvents():
      self.assertTrue(event.schema_match)

    # Test without original schema.
    del plugin.SCHEMAS[0]
    storage_writer = self._ParseDatabaseFileWithPlugin(
        [u'wal_database.db'], plugin, cache=cache, wal_path=wal_file)

    for event in storage_writer.GetEvents():
      if event.from_wal:
        self.assertTrue(event.schema_match)
      else:
        self.assertFalse(event.schema_match)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The time slice."""

class TimeSlice(object):
  """Time slice.

  The time slice is used to provide a context of events around an event
  of interest.

  Attributes:
    duration (int): duration of the time slize in minutes.
    event_timestamp (int): event timestamp of the time slice or None.
  """

  _MICRO_SECONDS_PER_MINUTE = 60 * 1000000

  def __init__(self, event_timestamp, duration=5):
    """Initializes the time slice.

    Args:
      event_timestamp (int): event timestamp of the time slice or None.
      duration (Optional[int]): duration of the time slize in minutes.
          The default is 5, which represent 2.5 minutes before and 2.5 minutes
          after the event timestamp.
    """
    super(TimeSlice, self).__init__()
    self.duration = duration
    self.event_timestamp = event_timestamp

  @property
  def end_timestamp(self):
    """int: slice end timestamp or None."""
    if self.event_timestamp:
      return self.event_timestamp + (
          self.duration * self._MICRO_SECONDS_PER_MINUTE)

  @property
  def start_timestamp(self):
    """int: slice start timestamp or None."""
    if self.event_timestamp:
      return self.event_timestamp - (
          self.duration * self._MICRO_SECONDS_PER_MINUTE)



# -*- coding: utf-8 -*-
"""The Basic Security Module (BSM) binary files event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class BSMFormatter(interface.ConditionalEventFormatter):
  """Formatter for an BSM log entry."""

  DATA_TYPE = u'bsm:event'

  FORMAT_STRING_PIECES = [
      u'Type: {event_type}',
      u'Information: {extra_tokens}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Type: {event_type}']

  SOURCE_LONG = u'BSM entry'
  SOURCE_SHORT = u'LOG'


class MacBSMFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Mac OS X BSM log entry."""

  DATA_TYPE = u'mac:bsm:event'

  FORMAT_STRING_PIECES = [
      u'Type: {event_type}',
      u'Return: {return_value}',
      u'Information: {extra_tokens}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Type: {event_type}',
      u'Return: {return_value}']

  SOURCE_LONG = u'BSM entry'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatters([
    BSMFormatter, MacBSMFormatter])



# -*- coding: utf-8 -*-
"""The Apple System Log Parser."""

import os

import construct

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import definitions
from plaso.lib import py2to3
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class ASLEventData(events.EventData):
  """Convenience class for an ASL event.

  Attributes:
    computer_name (str): name of the host.
    extra_information (str): extra fields associated to the event.
    facility (str): facility.
    group_id (int): group identifer (GID).
    level (str): level of criticality of the event.
    message_id (int): message identifier.
    message (str): message of the event.
    pid (int): process identifier (PID).
    read_uid (int): user identifier that can read this file, where -1
        represents all.
    read_gid (int): the group identifier that can read this file, where -1
        represents all.
    record_position (int): position of the event record.
    sender (str): sender or process that created the event.
    user_sid (str): user identifier (UID).
  """

  DATA_TYPE = u'mac:asl:event'

  def __init__(self):
    """Initializes event data."""
    super(ASLEventData, self).__init__(data_type=self.DATA_TYPE)
    self.computer_name = None
    self.extra_information = None
    self.facility = None
    self.group_id = None
    self.level = None
    self.message_id = None
    self.message = None
    self.pid = None
    self.read_gid = None
    self.read_uid = None
    self.record_position = None
    self.sender = None
    self.user_sid = None


class ASLParser(interface.FileObjectParser):
  """Parser for ASL log files."""

  NAME = u'asl_log'
  DESCRIPTION = u'Parser for ASL log files.'

  _ASL_SIGNATURE = b'ASL DB\x00\x00\x00\x00\x00\x00'

  # ASL File header.
  # signature: sequence of bytes that identifies an ASL file.
  # version: version of the file.
  # offset: first record in the file.
  # timestamp: time when the first entry was written.
  #     Contains the number of seconds since January 1, 1970 00:00:00 UTC.
  # last_offset: last record in the file.
  _ASL_HEADER_STRUCT = construct.Struct(
      u'asl_header_struct',
      construct.String(u'signature', 12),
      construct.UBInt32(u'version'),
      construct.UBInt64(u'offset'),
      construct.UBInt64(u'timestamp'),
      construct.UBInt32(u'cache_size'),
      construct.UBInt64(u'last_offset'),
      construct.Padding(36))

  # The record structure is:
  # [HEAP][STRUCTURE][4xExtraField][2xExtraField]*[PreviousEntry]
  # Record static structure.
  # tam_entry: it contains the number of bytes from this file position
  #            until the end of the record, without counts itself.
  # next_offset: next record. If is equal to 0x00, it is the last record.
  # asl_message_id: integer that has the numeric identification of the event.
  # timestamp: the entry creation date and time.
  #     Contains the number of seconds since January 1, 1970 00:00:00 UTC.
  # nanosecond: nanosecond to add to the timestamp.
  # level: level of priority.
  # pid: process identification that ask to save the record.
  # uid: user identification that has lunched the process.
  # gid: group identification that has lunched the process.
  # read_uid: identification id of a user. Only applied if is not -1 (all FF).
  #           Only root and this user can read the entry.
  # read_gid: the same than read_uid, but for the group.
  _ASL_RECORD_STRUCT = construct.Struct(
      u'asl_record_struct',
      construct.Padding(2),
      construct.UBInt32(u'tam_entry'),
      construct.UBInt64(u'next_offset'),
      construct.UBInt64(u'asl_message_id'),
      construct.UBInt64(u'timestamp'),
      construct.UBInt32(u'nanoseconds'),
      construct.UBInt16(u'level'),
      construct.UBInt16(u'flags'),
      construct.UBInt32(u'pid'),
      construct.UBInt32(u'uid'),
      construct.UBInt32(u'gid'),
      construct.UBInt32(u'read_uid'),
      construct.UBInt32(u'read_gid'),
      construct.UBInt64(u'ref_pid'))

  _ASL_RECORD_STRUCT_SIZE = _ASL_RECORD_STRUCT.sizeof()

  # 8-byte fields, they can be:
  # - String: [Nibble = 1000 (8)][Nibble = Length][7 Bytes = String].
  # - Integer: integer that has the byte position in the file that points
  #            to an ASL_RECORD_DYN_VALUE struct. If the value of the integer
  #            is equal to 0, it means that it has not data (skip).

  # If the field is a String, we use this structure to decode each
  # integer byte in the corresponding character (ASCII Char).
  _ASL_OCTET_STRING = construct.ExprAdapter(
      construct.Octet(u'string'),
      encoder=lambda obj, ctx: ord(obj),
      decoder=lambda obj, ctx: chr(obj))

  # Field string structure. If the first bit is 1, it means that it
  # is a String (1000) = 8, then the next nibble has the number of
  # characters. The last 7 bytes are the number of bytes.
  _ASL_STRING = construct.BitStruct(
      u'string',
      construct.Flag(u'type'),
      construct.Bits(u'filler', 3),
      construct.If(
          lambda ctx: ctx.type,
          construct.Nibble(u'string_length')),
      construct.If(
          lambda ctx: ctx.type,
          construct.Array(7, _ASL_OCTET_STRING)))

  # 8-byte pointer to a byte position in the file.
  _ASL_POINTER = construct.UBInt64(u'pointer')

  # Dynamic data structure pointed by a pointer that contains a String:
  # [2 bytes padding][4 bytes size of String][String].
  _ASL_RECORD_DYN_VALUE = construct.Struct(
      u'asl_record_dyn_value',
      construct.Padding(2),
      construct.UBInt32(u'size'),
      construct.Bytes(u'value', lambda ctx: ctx.size))

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(cls._ASL_SIGNATURE, offset=0)
    return format_specification

  def ParseFileObject(self, parser_mediator, file_object, **kwargs):
    """Parses an ASL file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      UnableToParseFile: when the file cannot be parsed.
    """
    try:
      header = self._ASL_HEADER_STRUCT.parse_stream(file_object)
    except (IOError, construct.FieldError) as exception:
      raise errors.UnableToParseFile(
          u'Unable to parse ASL Header with error: {0:s}.'.format(exception))

    if header.signature != self._ASL_SIGNATURE:
      raise errors.UnableToParseFile(u'Not an ASL Header, unable to parse.')

    offset = header.offset
    if not offset:
      return

    header_last_offset = header.last_offset

    previous_offset = offset
    event, offset = self.ReadASLEvent(parser_mediator, file_object, offset)
    while event:
      # Sanity check, the last read element must be the same as
      # indicated by the header.
      if offset == 0 and previous_offset != header_last_offset:
        parser_mediator.ProduceExtractionError(
            u'unable to parse header. Last element header does not match '
            u'header offset.')
      previous_offset = offset
      event, offset = self.ReadASLEvent(parser_mediator, file_object, offset)

  def ReadASLEvent(self, parser_mediator, file_object, offset):
    """Reads an ASL record at a specific offset.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
      offset (int): offset of the ASL record.

    Returns:
      tuple: contains:

        EventObject: event extracted from the ASL record or None.
        int: offset to the next ASL record in the file or None.
    """
    # The heap of the entry is saved to try to avoid seek (performance issue).
    # It has the real start position of the entry.
    dynamic_data_offset = file_object.tell()

    try:
      dynamic_data = file_object.read(offset - dynamic_data_offset)
    except IOError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to read ASL record dynamic data with error: {0:s}'.format(
              exception))
      return None, None

    if not offset:
      return None, None

    try:
      record_struct = self._ASL_RECORD_STRUCT.parse_stream(file_object)
    except (IOError, construct.FieldError) as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to parse ASL record with error: {0:s}'.format(exception))
      return None, None

    # Variable tam_fields = is the real length of the dynamic fields.
    # We have this: [Record_Struct] + [Dynamic_Fields] + [Pointer_Entry_Before]
    # In Record_Struct we have a field called tam_entry, where it has the number
    # of bytes until the end of the entry from the position that the field is.
    # The tam_entry is between the 2th and the 6th byte in the [Record_Struct].
    # tam_entry = ([Record_Struct]-6)+[Dynamic_Fields]+[Pointer_Entry_Before]
    # Also, we do not need [Point_Entry_Before] and then we delete the size of
    # [Point_Entry_Before] that it is 8 bytes (8):
    # tam_entry = ([Record_Struct]-6)+[Dynamic_Fields]+[Pointer_Entry_Before]
    # [Dynamic_Fields] = tam_entry - [Record_Struct] + 6 - 8
    # [Dynamic_Fields] = tam_entry - [Record_Struct] - 2
    tam_fields = record_struct.tam_entry - self._ASL_RECORD_STRUCT_SIZE - 2

    # Dynamic part of the entry that contains minimal four fields of 8 bytes
    # plus 2 x [8 bytes] fields for each extra ASL_Field.
    # The four first fields are always the Host, Sender, Facility and Message.
    # After the four first fields, the entry might have extra ASL_Fields.
    # For each extra ASL_field, it has a pair of 8-byte fields where the first
    # 8 bytes contains the name of the extra ASL_field and the second 8 bytes
    # contains the text of the extra field.
    # All of this 8-byte field can be saved using one of these three different
    # types:
    # - Null value ('0000000000000000'): nothing to do.
    # - String: It is string if first bit = 1 or first nibble = 8 (1000).
    #           Second nibble has the length of string.
    #           The next 7 bytes have the text characters of the string
    #           padding the end with null characters: '0x00'.
    #           Example: [8468 6964 6400 0000]
    #                    [8] String, [4] length, value: [68 69 64 64] = hidd.
    # - Pointer: static position in the file to a special struct
    #            implemented as an ASL_RECORD_DYN_VALUE.
    #            Example: [0000 0000 0000 0077]
    #            It points to the file position 0x077 that has a
    #            ASL_RECORD_DYN_VALUE structure.
    values = []
    while tam_fields > 0:
      try:
        field_data = file_object.read(8)
      except IOError as exception:
        parser_mediator.ProduceExtractionError(
            u'unable to read ASL field with error: {0:s}'.format(exception))
        return None, None

      # Try to read the field data as a string.
      try:
        asl_string_struct = self._ASL_STRING.parse(field_data)
        string_data = b''.join(
            asl_string_struct.string[0:asl_string_struct.string_length])
        values.append(string_data)

        # Go to parse the next extra field.
        tam_fields -= 8
        continue

      except ValueError:
        pass

      # If the field is not a string it must be a pointer.
      try:
        pointer_value = self._ASL_POINTER.parse(field_data)
      except ValueError as exception:
        parser_mediator.ProduceExtractionError(
            u'unable to parse ASL field with error: {0:s}'.format(exception))
        return None, None

      if not pointer_value:
        # Next extra field: 8 bytes more.
        tam_fields -= 8
        continue

      # The next IF ELSE is only for performance issues, avoiding seek.
      # If the pointer points a lower position than where the actual entry
      # starts, it means that it points to a previous entry.
      pos = pointer_value - dynamic_data_offset

      # Greater or equal 0 means that the data is in the actual entry.
      if pos >= 0:
        try:
          dyn_value_struct = self._ASL_RECORD_DYN_VALUE.parse(
              dynamic_data[pos:])
          dyn_value = dyn_value_struct.value.partition(b'\x00')[0]
          values.append(dyn_value)

        except (IOError, construct.FieldError) as exception:
          parser_mediator.ProduceExtractionError((
              u'unable to parse ASL record dynamic value with error: '
              u'{0:s}').format(exception))
          return None, None

      else:
        # Only if it is a pointer that points to the
        # heap from another entry we use the seek method.
        main_position = file_object.tell()

        # If the pointer is in a previous entry.
        if main_position > pointer_value:
          file_object.seek(pointer_value - main_position, os.SEEK_CUR)

          try:
            dyn_value_struct = self._ASL_RECORD_DYN_VALUE.parse_stream(
                file_object)
            dyn_value = dyn_value_struct.value.partition(b'\x00')[0]
            values.append(dyn_value)

          except (IOError, construct.FieldError):
            parser_mediator.ProduceExtractionError((
                u'the pointer at {0:d} (0x{0:08x}) points to invalid '
                u'information.').format(
                    main_position - self._ASL_POINTER.sizeof()))

          # Come back to the position in the entry.
          _ = file_object.read(main_position - file_object.tell())

        else:
          _ = file_object.read(pointer_value - main_position)

          dyn_value_struct = self._ASL_RECORD_DYN_VALUE.parse_stream(
              file_object)
          dyn_value = dyn_value_struct.value.partition(b'\x00')[0]
          values.append(dyn_value)

          # Come back to the position in the entry.
          file_object.seek(main_position - file_object.tell(), os.SEEK_CUR)

      # Next extra field: 8 bytes more.
      tam_fields -= 8

    # Read the last 8 bytes of the record that points to the previous entry.
    _ = file_object.read(8)

    # Parsing the dynamic values (text or pointers to position with text).
    # The first four are always the host, sender, facility, and message.
    number_of_values = len(values)
    if number_of_values < 4:
      parser_mediator.ProduceExtractionError(
          u'less than four values read from an ASL event.')

    computer_name = u'N/A'
    sender = u'N/A'
    facility = u'N/A'
    message = u'N/A'

    if number_of_values >= 1:
      computer_name = values[0].decode(u'utf-8')

    if number_of_values >= 2:
      sender = values[1].decode(u'utf-8')

    if number_of_values >= 3:
      facility = values[2].decode(u'utf-8')

    if number_of_values >= 4:
      message = values[3].decode(u'utf-8')

    # If the entry has an extra fields, they works as a pairs:
    # The first is the name of the field and the second the value.
    extra_information = u''
    if number_of_values > 4 and number_of_values % 2 == 0:
      # Taking all the extra attributes and merging them together,
      # e.g. a = [1, 2, 3, 4] will look like "1: 2, 3: 4".
      try:
        extra_values = map(py2to3.UNICODE_TYPE, values[4:])
        extra_information = u', '.join(
            map(u': '.join, zip(extra_values[0::2], extra_values[1::2])))
      except UnicodeDecodeError as exception:
        parser_mediator.ProduceExtractionError(
            u'unable to decode all ASL values in the extra information fields.')

    event_data = ASLEventData()
    event_data.computer_name = computer_name
    event_data.extra_information = extra_information
    event_data.facility = facility
    event_data.group_id = record_struct.gid
    event_data.level = record_struct.level
    event_data.message_id = record_struct.asl_message_id
    event_data.message = message
    event_data.pid = record_struct.pid
    event_data.read_gid = record_struct.read_gid
    event_data.read_uid = record_struct.read_uid
    event_data.record_position = offset
    event_data.sender = sender
    # Note that the user_sid value is expected to be a string.
    event_data.user_sid = u'{0:d}'.format(record_struct.uid)

    microseconds, _ = divmod(record_struct.nanoseconds, 1000)
    timestamp = (record_struct.timestamp * 1000000) + microseconds

    # TODO: replace by PosixTimeInNanoseconds.
    date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    return event, record_struct.next_offset


manager.ParsersManager.RegisterParser(ASLParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the hashers CLI arguments helper."""

from __future__ import unicode_literals

import argparse
import unittest

from plaso.cli import tools
from plaso.cli.helpers import hashers
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class HashersArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the hashers CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--hasher_file_size_limit SIZE] [--hashers HASHER_LIST]

Test argument parser.

optional arguments:
  --hasher_file_size_limit SIZE, --hasher-file-size-limit SIZE
                        Define the maximum file size in bytes that hashers
                        should process. Any larger file will be skipped. A
                        size of 0 represents no limit.
  --hashers HASHER_LIST
                        Define a list of hashers to use by the tool. This is a
                        comma separated list where each entry is the name of a
                        hasher, such as "md5,sha256". "all" indicates that all
                        hashers should be enabled. "none" disables all
                        hashers. Use "--hashers list" or "--info" to list the
                        available hashers.
"""

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py', description='Test argument parser.',
        add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    hashers.HashersArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.hashers = 'sha1'
    options.hasher_file_size_limit = 0

    test_tool = tools.CLITool()
    hashers.HashersArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._hasher_names_string, options.hashers)

    with self.assertRaises(errors.BadConfigObject):
      hashers.HashersArgumentsHelper.ParseOptions(options, None)

    with self.assertRaises(errors.BadConfigOption):
      options.hasher_file_size_limit = 'bogus'
      hashers.HashersArgumentsHelper.ParseOptions(options, test_tool)

    with self.assertRaises(errors.BadConfigOption):
      options.hasher_file_size_limit = -1
      hashers.HashersArgumentsHelper.ParseOptions(options, test_tool)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the storage factory class."""

from plaso.storage import sqlite_file as storage_sqlite_file
from plaso.storage import zip_file as storage_zip_file


class StorageFactory(object):
  """Storage factory."""

  @classmethod
  def CreateStorageFileForFile(cls, path):
    """Creates a storage file based on the file.

    Args:
      path (str): path to the storage file.

    Returns:
      StorageFile: a storage file or None if the storage file cannot be
          opened or the storage format is not supported.
    """
    if storage_sqlite_file.SQLiteStorageFile.CheckSupportedFormat(path):
      return storage_sqlite_file.SQLiteStorageFile()

    elif storage_zip_file.ZIPStorageFile.CheckSupportedFormat(path):
      return storage_zip_file.ZIPStorageFile()

  @classmethod
  def CreateStorageReaderForFile(cls, path):
    """Creates a storage reader based on the file.

    Args:
      path (str): path to the storage file.

    Returns:
      StorageReader: a storage reader or None if the storage file cannot be
          opened or the storage format is not supported.
    """
    if storage_sqlite_file.SQLiteStorageFile.CheckSupportedFormat(path):
      return storage_sqlite_file.SQLiteStorageFileReader(path)

    elif storage_zip_file.ZIPStorageFile.CheckSupportedFormat(path):
      return storage_zip_file.ZIPStorageFileReader(path)

  @classmethod
  def CreateStorageWriterForFile(cls, session, path):
    """Creates a storage writer based on the file.

    Args:
      session (Session): session the storage changes are part of.
      path (str): path to the storage file.

    Returns:
      StorageWriter: a storage writer or None if the storage file cannot be
          opened or the storage format is not supported.
    """
    if storage_sqlite_file.SQLiteStorageFile.CheckSupportedFormat(path):
      return storage_sqlite_file.SQLiteStorageFileWriter(session, path)

    elif storage_zip_file.ZIPStorageFile.CheckSupportedFormat(path):
      return storage_zip_file.ZIPStorageFileWriter(session, path)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Opera browser history parsers."""

import unittest

from plaso.formatters import opera  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import opera

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class OperaTypedParserTest(test_lib.ParserTestCase):
  """Tests for the Opera Typed History parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'typed_history.xml'])
  def testParse(self):
    """Tests the Parse function."""
    parser = opera.OperaTypedHistoryParser()
    storage_writer = self._ParseFile([u'typed_history.xml'], parser)

    self.assertEqual(storage_writer.number_of_events, 4)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-11 23:45:27')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.entry_selection, u'Filled from autocomplete.')

    expected_string = u'plaso.kiddaland.net (Filled from autocomplete.)'

    self._TestGetMessageStrings(event, expected_string, expected_string)

    event = events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-11 22:46:07')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.entry_selection, u'Manually typed.')

    expected_string = u'theonion.com (Manually typed.)'

    self._TestGetMessageStrings(event, expected_string, expected_string)


class OperaGlobalParserTest(test_lib.ParserTestCase):
  """Tests for the Opera Global History parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'global_history.dat'])
  def testParseFile(self):
    """Read a history file and run a few tests."""
    parser = opera.OperaGlobalHistoryParser()
    storage_writer = self._ParseFile([u'global_history.dat'], parser)

    self.assertEqual(storage_writer.number_of_events, 37)

    events = list(storage_writer.GetEvents())

    event = events[4]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-11 22:45:46')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'http://www.mbl.is/frettir/erlent/2013/11/11/'
        u'karl_bretaprins_faer_ellilifeyri/ (Karl Bretaprins fær ellilífeyri'
        u' - mbl.is) [First and Only Visit]')
    expected_short_message = (
        u'http://www.mbl.is/frettir/erlent/2013/11/11/'
        u'karl_bretaprins_faer_ellilifeyri/...')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[10]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-11 22:45:55')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[16]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-11 22:46:16')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_title = (
        u'10 Celebrities You Never Knew Were Abducted And Murdered '
        u'By Andie MacDowell | The Onion - America\'s Finest News Source')

    self.assertEqual(event.title, expected_title)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Output module for the "raw" (or native) Python format."""

from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import manager


class NativePythonFormatterHelper(object):
  """Helper for outputting as "raw" (or native) Python."""

  @classmethod
  def GetFormattedEventObject(cls, event_object):
    """Retrieves a string representation of the event object.

    Returns:
      A Unicode string containing the string representation of the event object.
    """
    time_string = timelib.Timestamp.CopyToIsoFormat(event_object.timestamp)

    lines_of_text = [
        u'+-' * 40,
        u'[Timestamp]:',
        u'  {0:s}'.format(time_string)]

    pathspec = getattr(event_object, u'pathspec', None)
    if pathspec:
      lines_of_text.append(u'[Pathspec]:')
      attribute_string = pathspec.comparable.replace(u'\n', u'\n  ')
      attribute_string = u'  {0:s}\n'.format(attribute_string)
      lines_of_text.append(attribute_string)

    # TODO: add support for event tag after event object clean up.

    lines_of_text.append(u'[Reserved attributes]:')
    out_additional = [u'[Additional attributes]:']

    for attribute_name, attribute_value in sorted(event_object.GetAttributes()):
      if attribute_name not in definitions.RESERVED_VARIABLE_NAMES:
        attribute_string = u'  {{{0!s}}} {1!s}'.format(
            attribute_name, attribute_value)
        out_additional.append(attribute_string)

      elif attribute_name not in (u'pathspec', u'tag'):
        attribute_string = u'  {{{0!s}}} {1!s}'.format(
            attribute_name, attribute_value)
        lines_of_text.append(attribute_string)

    lines_of_text.append(u'')
    out_additional.append(u'')

    lines_of_text.extend(out_additional)
    return u'\n'.join(lines_of_text)


class NativePythonOutputModule(interface.LinearOutputModule):
  """Output module for the "raw" (or native) Python output format."""

  NAME = u'rawpy'
  DESCRIPTION = u'"raw" (or native) Python output.'

  def WriteEventBody(self, event_object):
    """Writes the body of an event object to the output.

    Args:
      event_object: the event object (instance of EventObject).
    """
    output_string = NativePythonFormatterHelper.GetFormattedEventObject(
        event_object)
    self._WriteLine(output_string)


manager.OutputManager.RegisterOutput(NativePythonOutputModule)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the MSIECF event formatters."""

import unittest

from plaso.formatters import msiecf

from tests.formatters import test_lib


class MsiecfLeakFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the MSIECF leak item event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = msiecf.MsiecfLeakFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = msiecf.MsiecfLeakFormatter()

    expected_attribute_names = [
        u'cached_file_path',
        u'cached_file_size',
        u'recovered_string']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class MsiecfRedirectedFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the MSIECF redirected item event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = msiecf.MsiecfRedirectedFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = msiecf.MsiecfRedirectedFormatter()

    expected_attribute_names = [
        u'url',
        u'recovered_string']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


class MsiecfUrlFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the MSIECF URL item event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = msiecf.MsiecfUrlFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = msiecf.MsiecfUrlFormatter()

    expected_attribute_names = [
        u'url',
        u'number_of_hits',
        u'cached_file_path',
        u'cached_file_size',
        u'http_headers',
        u'recovered_string']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows firewall log parser."""

import unittest

from plaso.formatters import winfirewall  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import winfirewall

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class WinFirewallParserTest(test_lib.ParserTestCase):
  """Tests for the Windows firewall log parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'firewall.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser = winfirewall.WinFirewallParser()
    storage_writer = self._ParseFile([u'firewall.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 15)

    events = list(storage_writer.GetEvents())

    event = events[4]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2005-04-11 08:06:02')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.source_ip, u'123.45.78.90')
    self.assertEqual(event.dest_ip, u'123.156.78.90')

    event = events[7]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2005-04-11 08:06:26')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.size, 576)
    self.assertEqual(event.flags, u'A')
    self.assertEqual(event.tcp_ack, 987654321)

    expected_message = (
        u'DROP [ TCP RECEIVE ] '
        u'From: 123.45.78.90 :80 > 123.156.78.90 :1774 '
        u'Size (bytes): 576 '
        u'Flags [A] '
        u'TCP Seq Number: 123456789 '
        u'TCP ACK Number: 987654321 '
        u'TCP Window Size (bytes): 12345')
    expected_short_message = (
        u'DROP [TCP] 123.45.78.90 : 80 > 123.156.78.90 : 1774')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[9]

    self.assertEqual(event.icmp_type, 8)
    self.assertEqual(event.icmp_code, 0)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Safari Binary cookie event formatter."""

import unittest

from plaso.formatters import safari_cookies

from tests.formatters import test_lib


class SafaryCookieFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Safari Binary Cookie file entry event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = safari_cookies.SafaryCookieFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = safari_cookies.SafaryCookieFormatter()

    expected_attribute_names = [
        u'url',
        u'path',
        u'cookie_name',
        u'flags']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for the Google Chrome Cookie database."""

from dfdatetime import webkit_time as dfdatetime_webkit_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
# Register the cookie plugins.
from plaso.parsers import cookie_plugins  # pylint: disable=unused-import
from plaso.parsers import sqlite
from plaso.parsers.cookie_plugins import manager as cookie_plugins_manager
from plaso.parsers.sqlite_plugins import interface


class ChromeCookieEventData(events.EventData):
  """Chrome Cookie event data.

  Attributes:
    cookie_name (str): name of the cookie.
    hostname (str): hostname of host that set the cookie value.
    httponly (bool): True if the cookie cannot be accessed through client
        side script.
    path (str): path where the cookie got set.
    persistent (bool): True if the cookie is persistent.
    secure (bool): True if the cookie should only be transmitted over a
        secure channel.
    url (str): URL or path where the cookie got set.
    value (str): value of the cookie.
  """

  DATA_TYPE = u'chrome:cookie:entry'

  def __init__(self):
    """Initializes event data."""
    super(ChromeCookieEventData, self).__init__(data_type=self.DATA_TYPE)
    self.cookie_name = None
    self.data = None
    self.host = None
    self.httponly = None
    self.path = None
    self.persistent = None
    self.secure = None
    self.url = None


class ChromeCookiePlugin(interface.SQLitePlugin):
  """Parse Chrome Cookies file."""

  NAME = u'chrome_cookies'
  DESCRIPTION = u'Parser for Chrome cookies SQLite database files.'

  # Define the needed queries.
  QUERIES = [
      ((u'SELECT creation_utc, host_key, name, value, path, expires_utc, '
        u'secure, httponly, last_access_utc, has_expires, persistent '
        u'FROM cookies'), u'ParseCookieRow')]

  # The required tables common to Archived History and History.
  REQUIRED_TABLES = frozenset([u'cookies', u'meta'])

  SCHEMAS = [{
      u'cookies': (
          u'CREATE TABLE cookies (creation_utc INTEGER NOT NULL UNIQUE PRIMARY '
          u'KEY, host_key TEXT NOT NULL, name TEXT NOT NULL, value TEXT NOT '
          u'NULL, path TEXT NOT NULL, expires_utc INTEGER NOT NULL, secure '
          u'INTEGER NOT NULL, httponly INTEGER NOT NULL, last_access_utc '
          u'INTEGER NOT NULL, has_expires INTEGER DEFAULT 1, persistent '
          u'INTEGER DEFAULT 1)'),
      u'meta': (
          u'CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY, '
          u'value LONGVARCHAR)')}]

  # Point to few sources for URL information.
  URLS = [
      u'http://src.chromium.org/svn/trunk/src/net/cookies/',
      (u'http://www.dfinews.com/articles/2012/02/'
       u'google-analytics-cookies-and-forensic-implications')]

  # Google Analytics __utmz variable translation.
  # Taken from:
  #   http://www.dfinews.com/sites/dfinews.com/files/u739/Tab2Cookies020312.jpg
  GA_UTMZ_TRANSLATION = {
      u'utmcsr': u'Last source used to access.',
      u'utmccn': u'Ad campaign information.',
      u'utmcmd': u'Last type of visit.',
      u'utmctr': u'Keywords used to find site.',
      u'utmcct': u'Path to the page of referring link.'}

  def __init__(self):
    """Initializes a plugin."""
    super(ChromeCookiePlugin, self).__init__()
    self._cookie_plugins = (
        cookie_plugins_manager.CookiePluginsManager.GetPlugins())

  def ParseCookieRow(self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a cookie row.

    Args:
      parser_mediator (ParserMediator): parser mediator.
      row (sqlite3.Row): row resulting from the query.
      query (Optional[str]): query string.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    cookie_name = row['name']
    cookie_data = row['value']

    hostname = row['host_key']
    if hostname.startswith('.'):
      hostname = hostname[1:]

    if row['secure']:
      scheme = u'https'
    else:
      scheme = u'http'

    url = u'{0:s}://{1:s}{2:s}'.format(scheme, hostname, row['path'])

    event_data = ChromeCookieEventData()
    event_data.cookie_name = cookie_name
    event_data.data = cookie_data
    event_data.host = hostname
    event_data.httponly = True if row['httponly'] else False
    event_data.path = row['path']
    event_data.persistent = True if row['persistent'] else False
    event_data.query = query
    event_data.secure = True if row['secure'] else False
    event_data.url = url

    timestamp = row['creation_utc']
    date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    timestamp = row['last_access_utc']
    date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_LAST_ACCESS)
    parser_mediator.ProduceEventWithEventData(event, event_data)

    if row['has_expires']:
      timestamp = row['expires_utc']
      date_time = dfdatetime_webkit_time.WebKitTime(timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, u'Cookie Expires')
      parser_mediator.ProduceEventWithEventData(event, event_data)

    for plugin in self._cookie_plugins:
      if cookie_name != plugin.COOKIE_NAME:
        continue

      try:
        plugin.UpdateChainAndProcess(
            parser_mediator, cookie_data=cookie_data, cookie_name=cookie_name,
            url=url)

      except Exception as exception:  # pylint: disable=broad-except
        parser_mediator.ProduceExtractionError(
            u'plugin: {0:s} unable to parse cookie with error: {1:s}'.format(
                plugin.NAME, exception))


sqlite.SQLiteParser.RegisterPlugin(ChromeCookiePlugin)



# -*- coding: utf-8 -*-
"""Imports for the output (module) manager."""

from plaso.output import dynamic
from plaso.output import elastic
from plaso.output import json_line
from plaso.output import json_out
from plaso.output import kml
from plaso.output import l2t_csv
from plaso.output import mysql_4n6time
from plaso.output import null
from plaso.output import rawpy
from plaso.output import sqlite_4n6time
from plaso.output import timesketch_out
from plaso.output import tln
from plaso.output import xlsx



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This file contains tests for Services Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import services

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class ServicesRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Services Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        u'TestDriver', key_path=key_path, last_written_time=filetime.timestamp,
        offset=1456)

    value_data = b'\x02\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'Type', data=value_data, data_type=dfwinreg_definitions.REG_DWORD,
        offset=123)
    registry_key.AddValue(registry_value)

    value_data = b'\x02\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'Start', data=value_data, data_type=dfwinreg_definitions.REG_DWORD,
        offset=127)
    registry_key.AddValue(registry_value)

    value_data = b'\x01\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ErrorControl', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD, offset=131)
    registry_key.AddValue(registry_value)

    value_data = u'Pnp Filter'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'Group', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=140)
    registry_key.AddValue(registry_value)

    value_data = u'Test Driver'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'DisplayName', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=160)
    registry_key.AddValue(registry_value)

    value_data = u'testdriver.inf_x86_neutral_dd39b6b0a45226c4'.encode(
        u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'DriverPackageId', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=180)
    registry_key.AddValue(registry_value)

    value_data = u'C:\\Dell\\testdriver.sys'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ImagePath', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=200)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function on a virtual key."""
    key_path = (
        u'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\services\\TestDriver')
    time_string = u'2012-08-28 09:23:49.002031'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = services.ServicesPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    self.assertEqual(storage_writer.number_of_events, 1)

    events = list(storage_writer.GetEvents())

    event = events[0]

    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'DisplayName: Test Driver '
        u'DriverPackageId: testdriver.inf_x86_neutral_dd39b6b0a45226c4 '
        u'ErrorControl: Normal (1) '
        u'Group: Pnp Filter '
        u'ImagePath: C:\\Dell\\testdriver.sys '
        u'Start: Auto Start (2) '
        u'Type: File System Driver (0x2)').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testProcessFile(self):
    """Tests the Process function on a key in a file."""
    test_file_entry = self._GetTestFileEntry([u'SYSTEM'])
    key_path = u'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\services'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    events = []

    # Select a few service subkeys to perform additional testing.
    bits_events = None
    mc_task_manager_events = None
    rdp_video_miniport_events = None

    plugin = services.ServicesPlugin()
    for winreg_subkey in registry_key.GetSubkeys():
      storage_writer = self._ParseKeyWithPlugin(
          winreg_subkey, plugin, file_entry=test_file_entry)

      events_subkey = list(storage_writer.GetEvents())

      events.extend(list(events_subkey))

      if winreg_subkey.name == u'BITS':
        bits_events = events_subkey
      elif winreg_subkey.name == u'McTaskManager':
        mc_task_manager_events = events_subkey
      elif winreg_subkey.name == u'RdpVideoMiniport':
        rdp_video_miniport_events = events_subkey

    self.assertEqual(len(events), 416)

    # Test the BITS subkey events.
    self.assertEqual(len(bits_events), 1)

    event = bits_events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-06 20:43:27.639075')
    self.assertEqual(event.timestamp, expected_timestamp)

    self._TestRegvalue(event, u'Type', 0x20)
    self._TestRegvalue(event, u'Start', 3)
    self._TestRegvalue(
        event, u'ServiceDll', u'%SystemRoot%\\System32\\qmgr.dll')

    # Test the McTaskManager subkey events.
    self.assertEqual(len(mc_task_manager_events), 1)

    event = mc_task_manager_events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2011-09-16 20:49:16.877415')
    self.assertEqual(event.timestamp, expected_timestamp)

    self._TestRegvalue(event, u'DisplayName', u'McAfee Task Manager')
    self._TestRegvalue(event, u'Type', 0x10)

    # Test the RdpVideoMiniport subkey events.
    self.assertEqual(len(rdp_video_miniport_events), 1)

    event = rdp_video_miniport_events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2011-09-17 13:37:59.347157')
    self.assertEqual(event.timestamp, expected_timestamp)

    self._TestRegvalue(event, u'Start', 3)
    expected_value = u'System32\\drivers\\rdpvideominiport.sys'
    self._TestRegvalue(event, u'ImagePath', expected_value)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Zeitgeist event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class ZeitgeistFormatter(interface.EventFormatter):
  """Formatter for a Zeitgeist activity database event."""

  DATA_TYPE = u'zeitgeist:activity'

  FORMAT_STRING = u'{subject_uri}'

  SOURCE_LONG = u'Zeitgeist activity log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(ZeitgeistFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the unique hashes analysis plugin."""

import unittest

from dfvfs.path import fake_path_spec

from plaso.analysis import file_hashes

from tests.analysis import test_lib


class UniqueHashesTest(test_lib.AnalysisPluginTestCase):
  """Test for the unique hashes analysis plugin."""

  _TEST_EVENTS = [
      {u'pathspec': fake_path_spec.FakePathSpec(
          location=u'/var/testing directory with space/file.txt'),
       u'test_hash': u'4'},
      {u'pathspec': fake_path_spec.FakePathSpec(
          location=u'C:\\Windows\\a.file.txt'),
       u'test_hash': u'4'},
      {u'pathspec': fake_path_spec.FakePathSpec(
          location=u'/opt/dfvfs'),
       u'test_hash': u'4'},
      {u'pathspec': fake_path_spec.FakePathSpec(
          location=u'/opt/2hash_file'),
       u'test_hash': u'4',
       u'alternate_test_hash': u'5'},
      {u'pathspec': fake_path_spec.FakePathSpec(
          location=u'/opt/no_hash_file')}
  ]

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    events = []
    for event_dictionary in self._TEST_EVENTS:
      event = self._CreateTestEventObject(event_dictionary)
      events.append(event)

    plugin = file_hashes.FileHashesPlugin()
    storage_writer = self._AnalyzeEvents(events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)

    analysis_report = storage_writer.analysis_reports[0]

    expected_text = (
        u'Listing file paths and hashes\n'
        u'FAKE:/opt/2hash_file: alternate_test_hash=5 test_hash=4\n'
        u'FAKE:/opt/dfvfs: test_hash=4\n'
        u'FAKE:/opt/no_hash_file:\n'
        u'FAKE:/var/testing directory with space/file.txt: test_hash=4\n'
        u'FAKE:C:\\Windows\\a.file.txt: test_hash=4\n')

    self.assertEqual(expected_text, analysis_report.text)
    self.assertEqual(analysis_report.plugin_name, u'file_hashes')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the SAM users Windows Registry event formatter."""

import unittest

from plaso.formatters import sam_users

from tests.formatters import test_lib


class SAMUsersWindowsRegistryEventFormatterTest(
    test_lib.EventFormatterTestCase):
  """Tests for the SAM users Windows Registry event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = sam_users.SAMUsersWindowsRegistryEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = sam_users.SAMUsersWindowsRegistryEventFormatter()

    expected_attribute_names = [
        u'account_rid',
        u'comments',
        u'fullname',
        u'key_path',
        u'login_count',
        u'username']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the event formatters manager class."""

import logging

from plaso.formatters import default


class FormattersManager(object):
  """Class that implements the formatters manager."""

  _formatter_classes = {}
  _formatter_objects = {}

  @classmethod
  def DeregisterFormatter(cls, formatter_class):
    """Deregisters a formatter class.

    The formatter classes are identified based on their lower case data type.

    Args:
      formatter_class (type): class of the formatter.

    Raises:
      KeyError: if formatter class is not set for the corresponding data type.
    """
    formatter_data_type = formatter_class.DATA_TYPE.lower()
    if formatter_data_type not in cls._formatter_classes:
      raise KeyError(
          u'Formatter class not set for data type: {0:s}.'.format(
              formatter_class.DATA_TYPE))

    del cls._formatter_classes[formatter_data_type]

  @classmethod
  def GetFormatterObject(cls, data_type):
    """Retrieves the formatter object for a specific data type.

    Args:
      data_type (str): data type.

    Returns:
      EventFormatter: corresponding formatter or the default formatter if
          not available.
    """
    data_type = data_type.lower()
    if data_type not in cls._formatter_objects:
      formatter_object = None

      if data_type in cls._formatter_classes:
        formatter_class = cls._formatter_classes[data_type]
        # TODO: remove the need to instantiate the Formatter classes
        # and use class methods only.
        formatter_object = formatter_class()

      if not formatter_object:
        logging.warning(
            u'Using default formatter for data type: {0:s}'.format(data_type))
        formatter_object = default.DefaultFormatter()

      cls._formatter_objects[data_type] = formatter_object

    return cls._formatter_objects[data_type]

  @classmethod
  def GetMessageStrings(cls, formatter_mediator, event):
    """Retrieves the formatted message strings for a specific event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      list[str, str]: long and short version of the message string.
    """
    formatter_object = cls.GetFormatterObject(event.data_type)
    return formatter_object.GetMessages(formatter_mediator, event)

  @classmethod
  def GetSourceStrings(cls, event):
    """Retrieves the formatted source strings for a specific event object.

    Args:
      event (EventObject): event.

    Returns:
      list[str, str]: short and long version of the source of the event.
    """
    # TODO: change this to return the long variant first so it is consistent
    # with GetMessageStrings.
    formatter_object = cls.GetFormatterObject(event.data_type)
    return formatter_object.GetSources(event)

  @classmethod
  def RegisterFormatter(cls, formatter_class):
    """Registers a formatter class.

    The formatter classes are identified based on their lower case data type.

    Args:
      formatter_class (type): class of the formatter.

    Raises:
      KeyError: if formatter class is already set for the corresponding
                data type.
    """
    formatter_data_type = formatter_class.DATA_TYPE.lower()
    if formatter_data_type in cls._formatter_classes:
      raise KeyError((
          u'Formatter class already set for data type: {0:s}.').format(
              formatter_class.DATA_TYPE))

    cls._formatter_classes[formatter_data_type] = formatter_class

  @classmethod
  def RegisterFormatters(cls, formatter_classes):
    """Registers formatter classes.

    The formatter classes are identified based on their lower case data type.

    Args:
      formatter_classes (list[type]): classes of the formatters.

    Raises:
      KeyError: if formatter class is already set for the corresponding
                data type.
    """
    for formatter_class in formatter_classes:
      cls.RegisterFormatter(formatter_class)



# -*- coding: utf-8 -*-
"""Install history plist plugin."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import plist_event
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class InstallHistoryPlugin(interface.PlistPlugin):
  """Plist plugin that extracts the installation history."""

  NAME = u'macosx_install_history'
  DESCRIPTION = u'Parser for installation history plist files.'

  PLIST_PATH = u'InstallHistory.plist'
  PLIST_KEYS = frozenset([
      u'date', u'displayName', u'displayVersion', u'processName',
      u'packageIdentifiers'])

  def GetEntries(self, parser_mediator, top_level=None, **unused_kwargs):
    """Extracts relevant install history entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      top_level (dict[str, object]): plist top-level key.
    """
    for entry in top_level:
      datetime_value = entry.get(u'date', None)
      package_identifiers = entry.get(u'packageIdentifiers', [])

      if not datetime_value or not package_identifiers:
        continue

      display_name = entry.get(u'displayName', u'<UNKNOWN>')
      display_version = entry.get(u'displayVersion', u'<DISPLAY_VERSION>')
      process_name = entry.get(u'processName', u'<PROCESS_NAME>')
      package_identifiers = u', '.join(package_identifiers)

      event_data = plist_event.PlistTimeEventData()
      event_data.desc = (
          u'Installation of [{0:s} {1:s}] using [{2:s}]. Packages: '
          u'{3:s}.').format(
              display_name, display_version, process_name, package_identifiers)
      event_data.key = u''
      event_data.root = u'/item'

      timestamp = timelib.Timestamp.FromPythonDatetime(datetime_value)
      date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
          timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_WRITTEN)
      parser_mediator.ProduceEventWithEventData(event, event_data)


plist.PlistParser.RegisterPlugin(InstallHistoryPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Microsoft IIS log file event formatter."""

import unittest

from plaso.formatters import iis

from tests.formatters import test_lib


class IISLogFileEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Microsoft IIS log file event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = iis.IISLogFileEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = iis.IISLogFileEventFormatter()

    expected_attribute_names = [
        u'http_method', u'requested_uri_stem', u'source_ip', u'dest_ip',
        u'dest_port', u'http_status', u'sent_bytes', u'received_bytes',
        u'user_agent', u'protocol_version']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the UserAssist Windows Registry plugin."""

import unittest

from plaso.formatters import userassist  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import userassist

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class UserAssistPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the UserAssist Windows Registry plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER.DAT'])
  def testProcessOnWinXP(self):
    """Tests the Process function on a Windows XP Registry file."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\UserAssist\\{75048700-EF1F-11D0-9888-006097DEACF9}')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = userassist.UserAssistPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 14)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-08-04 15:11:22.811067')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_WRITTEN)

    expected_value_name = u'UEME_RUNPIDL:%csidl2%\\MSN.lnk'
    self.assertEqual(event.value_name, expected_value_name)
    self.assertEqual(event.number_of_executions, 14)

    expected_message = (
        u'[{0:s}\\Count] '
        u'Value name: {1:s} '
        u'Count: 14').format(key_path, expected_value_name)
    expected_short_message = u'{0:s} Count: 14'.format(expected_value_name)

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcessOnWin7(self):
    """Tests the Process function on a Windows 7 Registry file."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\UserAssist\\{CEBFF5CD-ACE2-4F4F-9178-9926F41749EA}')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = userassist.UserAssistPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 61)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_WRITTEN)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-11-10 07:49:37.078067')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_value_name = u'Microsoft.Windows.GettingStarted'
    self.assertEqual(event.value_name, expected_value_name)
    self.assertEqual(event.number_of_executions, 14)
    self.assertEqual(event.application_focus_count, 21)
    self.assertEqual(event.application_focus_duration, 420000)

    expected_message = (
        u'[{0:s}\\Count] '
        u'UserAssist entry: 1 '
        u'Value name: {1:s} '
        u'Count: 14 '
        u'Application focus count: 21 '
        u'Application focus duration: 420000').format(
            key_path, expected_value_name)
    expected_short_message = u'{0:s} Count: 14'.format(expected_value_name)

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the MountPoints2 Windows Registry plugin."""

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import mountpoints

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


class MountPoints2PluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the MountPoints2 Windows Registry plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\MountPoints2')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = mountpoints.MountPoints2Plugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    self.assertEqual(storage_writer.number_of_events, 5)

    events = list(storage_writer.GetEvents())

    event = events[0]

    self.assertEqual(event.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2011-08-23 17:10:14.960960')
    self.assertEqual(event.timestamp, expected_timestamp)

    regvalue = event.regvalue
    self.assertEqual(regvalue.get(u'Share_Name'), u'\\home\\nfury')

    expected_message = (
        u'[{0:s}] Label: Home Drive Remote_Server: controller Share_Name: '
        u'\\home\\nfury Type: Remote Drive Volume: '
        u'##controller#home#nfury').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The fake storage intended for testing."""

import copy

from plaso.lib import definitions
from plaso.storage import event_heaps
from plaso.storage import identifiers
from plaso.storage import interface


class FakeStorageWriter(interface.StorageWriter):
  """Fake storage writer object.

  Attributes:
    analysis_reports (list[AnalysisReport]): analysis reports.
    errors (list[ExtractionError]): extraction errors.
    session_completion (SessionCompletion): session completion attribute
        container.
    session_start (SessionStart): session start attribute container.
    task_completion (TaskCompletion): task completion attribute container.
    task_start (TaskStart): task start attribute container.
  """

  def __init__(
      self, session, storage_type=definitions.STORAGE_TYPE_SESSION, task=None):
    """Initializes a storage writer object.

    Args:
      session (Session): session the storage changes are part of.
      storage_type (Optional[str]): storage type.
      task(Optional[Task]): task.
    """
    super(FakeStorageWriter, self).__init__(
        session, storage_type=storage_type, task=task)
    self._errors = []
    self._event_data = {}
    self._event_sources = []
    self._event_tags = []
    self._events = []
    self._is_open = False
    self._task_storage_writers = {}
    self.analysis_reports = []
    self.session_completion = None
    self.session_start = None
    self.task_completion = None
    self.task_start = None

  def _PrepareAttributeContainer(self, attribute_container):
    """Prepares an attribute container for storage.

    Args:
      attribute_container (AttributeContainer): attribute container.

    Returns:
      AttributeContainer: copy of the attribute container to store in
          the fake storage.
    """
    attribute_values_hash = hash(attribute_container.GetAttributeValuesString())
    identifier = identifiers.FakeIdentifier(attribute_values_hash)
    attribute_container.SetIdentifier(identifier)

    # Make sure the fake storage preserves the state of the attribute container.
    return copy.deepcopy(attribute_container)

  def _RaiseIfNotWritable(self):
    """Raises if the storage file is not writable.

    Raises:
      IOError: when the storage writer is closed.
    """
    if not self._is_open:
      raise IOError(u'Unable to write to closed storage writer.')

  def AddAnalysisReport(self, analysis_report):
    """Adds an analysis report.

    Args:
      analysis_report (AnalysisReport): analysis report.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    analysis_report = self._PrepareAttributeContainer(analysis_report)

    self.analysis_reports.append(analysis_report)

  def AddError(self, error):
    """Adds an error.

    Args:
      error (ExtractionError): error.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    error = self._PrepareAttributeContainer(error)

    self._errors.append(error)
    self.number_of_errors += 1

  def AddEvent(self, event):
    """Adds an event.

    Args:
      event (EventObject): event.

    Raises:
      IOError: when the storage writer is closed or
          if the event data identifier type is not supported.
    """
    self._RaiseIfNotWritable()

    # TODO: change to no longer allow event_data_identifier is None
    # after refactoring every parser to generate event data.
    event_data_identifier = event.GetEventDataIdentifier()
    if event_data_identifier:
      if not isinstance(event_data_identifier, identifiers.FakeIdentifier):
        raise IOError(u'Unsupported event data identifier type: {0:s}'.format(
            type(event_data_identifier)))

    event = self._PrepareAttributeContainer(event)

    self._events.append(event)
    self.number_of_events += 1

  def AddEventData(self, event_data):
    """Adds event data.

    Args:
      event_data (EventData): event data.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    event_data = self._PrepareAttributeContainer(event_data)

    identifier = event_data.GetIdentifier()
    lookup_key = identifier.CopyToString()
    self._event_data[lookup_key] = event_data

  def AddEventSource(self, event_source):
    """Adds an event source.

    Args:
      event_source (EventSource): event source.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    event_source = self._PrepareAttributeContainer(event_source)

    self._event_sources.append(event_source)
    self.number_of_event_sources += 1

  def AddEventTag(self, event_tag):
    """Adds an event tag.

    Args:
      event_tag (EventTag): event tag.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    event_identifier = event_tag.GetEventIdentifier()
    if not isinstance(event_identifier, identifiers.FakeIdentifier):
      raise IOError(u'Unsupported event identifier type: {0:s}'.format(
          type(event_identifier)))

    event_tag = self._PrepareAttributeContainer(event_tag)

    self._event_tags.append(event_tag)
    self.number_of_event_tags += 1

  def CreateTaskStorage(self, task):
    """Creates a task storage.

    Args:
      task (Task): task.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      IOError: if the task storage already exists.
    """
    if task.identifier in self._task_storage_writers:
      raise IOError(u'Storage writer for task: {0:s} already exists.'.format(
          task.identifier))

    storage_writer = FakeStorageWriter(
        self._session, storage_type=definitions.STORAGE_TYPE_TASK, task=task)
    self._task_storage_writers[task.identifier] = storage_writer
    return storage_writer

  def Close(self):
    """Closes the storage writer.

    Raises:
      IOError: when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    self._is_open = False

  def GetErrors(self):
    """Retrieves the errors.

    Returns:
      generator(ExtractionError): error generator.
    """
    return iter(self._errors)

  def GetEvents(self):
    """Retrieves the events.

    Yields:
      EventObject: event.
    """
    return iter(self._events)

  def GetEventData(self):
    """Retrieves the event data.

    Returns:
      generator(EventData): event data generator.
    """
    return iter(self._event_data.values())

  def GetEventSources(self):
    """Retrieves the event sources.

    Returns:
      generator(EventSource): event source generator.
    """
    return iter(self._event_sources)

  def GetEventTags(self):
    """Retrieves the event tags.

    Returns:
      generator(EventTags): event tag generator.
    """
    return iter(self._event_tags)

  def GetFirstWrittenEventSource(self):
    """Retrieves the first event source that was written after open.

    Using GetFirstWrittenEventSource and GetNextWrittenEventSource newly
    added event sources can be retrieved in order of addition.

    Returns:
      EventSource: event source or None if there are no newly written ones.

    Raises:
      IOError: when the storage writer is closed.
    """
    if not self._is_open:
      raise IOError(u'Unable to read from closed storage writer.')

    if self._written_event_source_index >= len(self._event_sources):
      return

    event_source = self._event_sources[self._first_written_event_source_index]
    self._written_event_source_index = (
        self._first_written_event_source_index + 1)
    return event_source

  def GetNextWrittenEventSource(self):
    """Retrieves the next event source that was written after open.

    Returns:
      EventSource: event source or None if there are no newly written ones.

    Raises:
      IOError: when the storage writer is closed.
    """
    if not self._is_open:
      raise IOError(u'Unable to read from closed storage writer.')

    if self._written_event_source_index >= len(self._event_sources):
      return

    event_source = self._event_sources[self._written_event_source_index]
    self._written_event_source_index += 1
    return event_source

  def GetSortedEvents(self, time_range=None):
    """Retrieves the events in increasing chronological order.

    Args:
      time_range (Optional[TimeRange]): time range used to filter events
          that fall in a specific period.

    Returns:
      generator(EventObject): event generator.

    Raises:
      IOError: when the storage writer is closed.
    """
    if not self._is_open:
      raise IOError(u'Unable to read from closed storage writer.')

    event_heap = event_heaps.EventHeap()

    for event in self._events:
      if (time_range and (
          event.timestamp < time_range.start_timestamp or
          event.timestamp > time_range.end_timestamp)):
        continue

      event_heap.PushEvent(event)

    return iter(event_heap.PopEvents())

  def Open(self):
    """Opens the storage writer.

    Raises:
      IOError: if the storage writer is already opened.
    """
    if self._is_open:
      raise IOError(u'Storage writer already opened.')

    self._is_open = True

    self._first_written_event_source_index = len(self._event_sources)
    self._written_event_source_index = self._first_written_event_source_index

  def PrepareMergeTaskStorage(self, task):
    """Prepares a task storage for merging.

    Args:
      task (Task): task.

    Raises:
      IOError: if the task storage does not exist.
    """
    if task.identifier not in self._task_storage_writers:
      raise IOError(u'Storage writer for task: {0:s} does not exist.'.format(
          task.identifier))

  def ReadPreprocessingInformation(self, unused_knowledge_base):
    """Reads preprocessing information.

    The preprocessing information contains the system configuration which
    contains information about various system specific configuration data,
    for example the user accounts.

    Args:
      knowledge_base (KnowledgeBase): is used to store the preprocessing
          information.

    Raises:
      IOError: if the storage type does not support writing preprocessing
          information or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_SESSION:
      raise IOError(u'Preprocessing information not supported by storage type.')

    # TODO: implement.

  def SetSerializersProfiler(self, serializers_profiler):
    """Sets the serializers profiler.

    Args:
      serializers_profiler (SerializersProfiler): serializers profile.
    """
    pass

  def WritePreprocessingInformation(self, unused_knowledge_base):
    """Writes preprocessing information.

    Args:
      knowledge_base (KnowledgeBase): contains the preprocessing information.

    Raises:
      IOError: if the storage type does not support writing preprocessing
          information or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_SESSION:
      raise IOError(u'Preprocessing information not supported by storage type.')

    # TODO: implement.

  def WriteSessionCompletion(self, aborted=False):
    """Writes session completion information.

    Args:
      aborted (Optional[bool]): True if the session was aborted.

    Raises:
      IOError: if the storage type does not support writing a session
          completion or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_SESSION:
      raise IOError(u'Session start not supported by storage type.')

    self._session.aborted = aborted
    self.session_completion = self._session.CreateSessionCompletion()

  def WriteSessionStart(self):
    """Writes session start information.

    Raises:
      IOError: if the storage type does not support writing a session
          start or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_SESSION:
      raise IOError(u'Session start not supported by storage type.')

    self.session_start = self._session.CreateSessionStart()

  def WriteTaskCompletion(self, aborted=False):
    """Writes task completion information.

    Args:
      aborted (Optional[bool]): True if the session was aborted.

    Raises:
      IOError: if the storage type does not support writing a task
          completion or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_TASK:
      raise IOError(u'Task completion not supported by storage type.')

    self._task.aborted = aborted
    self.task_completion = self._task.CreateTaskCompletion()

  def WriteTaskStart(self):
    """Writes task start information.

    Raises:
      IOError: if the storage type does not support writing a task
          start or when the storage writer is closed.
    """
    self._RaiseIfNotWritable()

    if self._storage_type != definitions.STORAGE_TYPE_TASK:
      raise IOError(u'Task start not supported by storage type.')

    self.task_start = self._task.CreateTaskStart()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the File History ESE database file."""

import unittest

from plaso.formatters import file_history  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers.esedb_plugins import file_history

from tests import test_lib as shared_test_lib
from tests.parsers.esedb_plugins import test_lib


class FileHistoryESEDBPluginTest(test_lib.ESEDBPluginTestCase):
  """Tests for the File History ESE database plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'Catalog1.edb'])
  def testProcess(self):
    """Tests the Process function."""
    plugin = file_history.FileHistoryESEDBPlugin()
    storage_writer = self._ParseESEDBFileWithPlugin(
        [u'Catalog1.edb'], plugin)

    self.assertEqual(storage_writer.number_of_events, 2713)

    events = list(storage_writer.GetEvents())

    event = events[702]

    self.assertEqual(event.usn_number, 9251162904)
    self.assertEqual(event.identifier, 356)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-10-12 17:34:36.688580')

    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_MODIFICATION)

    filename = u'?UP\\Favorites\\Links\\Lenovo'
    self.assertEqual(event.original_filename, filename)

    expected_message = (
        u'Filename: {0:s} '
        u'Identifier: 356 '
        u'Parent Identifier: 230 '
        u'Attributes: 16 '
        u'USN number: 9251162904').format(filename)

    expected_short_message = u'Filename: {0:s}'.format(filename)

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the language CLI arguments helper."""

import argparse
import unittest

from plaso.cli import tools
from plaso.cli.helpers import language
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class LanguagergumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the language CLI arguments helper."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--language LANGUAGE]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      (u'  --language LANGUAGE  The preferred language identifier for Windows '
       u'Event Log'),
      (u'                       message strings. Use "--language list" to see '
       u'a list of'),
      (u'                       available language identifiers. Note that '
       u'formatting'),
      (u'                       will fall back on en-US (LCID 0x0409) if the '
       u'preferred'),
      (u'                       language is not available in the database of '
       u'message'),
      u'                       string templates.',
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py', description=u'Test argument parser.',
        add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    language.LanguageArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.preferred_language = u'is'

    test_tool = tools.CLITool()
    language.LanguageArgumentsHelper.ParseOptions(options, test_tool)

    self.assertEqual(test_tool._preferred_language, options.preferred_language)

    with self.assertRaises(errors.BadConfigObject):
      language.LanguageArgumentsHelper.ParseOptions(options, None)


if __name__ == '__main__':
  unittest.main()
